package DE_EPAGES::Amazon::API::Object::RemoteMWSProduct;
use base DE_EPAGES::Amazon::API::Object::RemoteAWS;

use DateTime;
use XML::Simple;
use Encode qw( decode_utf8 );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);

use strict;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'InvalidParameterValue'  => "Invalid parameter value by call to Amazon API: #Message",
    };
}

#ERRORDEFINITION END

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = __PACKAGE__->SUPER::new(%args);

    for (qw/MarketplaceID SellerID MWSAuthToken/) {
        Error( 'MissingConstructorArgument', { 'Argument' => $_ } ) unless ( $args{$_} );
    }

    $self->{sellerID}      = $args{'SellerID'};
    $self->{mwsAuthToken}  = $args{'MWSAuthToken'};
    $self->{marketplaceID} = $args{'MarketplaceID'};
    $self->{'PortalURL'}   = $args{'PortalURL'};
    # mws uses a 120sec timeout.
    $self->userAgent->timeout(GetDefaultTimeOut); ## no critic qw(:ProhibitMagicNumbers)
    return $self;
}

sub ListMatchingProducts {
    my $self = shift;
    my (%args) = @_;
    LogDebug( 'Debug:', %args );

    my %request = (
        'Action'            => 'ListMatchingProducts',
        'SellerId'          => $self->sellerID,
        'MWSAuthToken'      => $self->mwsAuthToken,
        'SignatureVersion'  => '2',
        'Version'           => '2011-10-01',
        'MarketplaceId'     => $self->marketplaceID,
    );

    foreach my $i ( keys %args ) {
        $request{$i} = $args{$i};
    }
    $self->replaceUmlaute(\%request);
    return $self->query(\%request);
}

sub GetMatchingProduct {
    my $self = shift;
    my (%args) = @_;
    # build ASIN list in format ASINList.ASIN.1,ASINList.ASIN.2...
    my %ASINList;
    unless ( defined $args{'ASINs'} ){
        Error('XMLFileError', {
            'Reason'  =>, 'no incomming ASIN',
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }
    my $increment = 1;
    foreach my $VariationASIN ($args{'ASINs'}){
        $ASINList{'ASINList.ASIN.'.$increment++} = $VariationASIN;
    }
    my %request = (
        %ASINList,
        'Action'            => 'GetMatchingProduct',
        'SellerId'          => $self->sellerID,
        'MWSAuthToken'      => $self->mwsAuthToken,
        'SignatureVersion'  => '2',
        'Version'           => '2011-10-01',
        'MarketplaceId'     => $self->marketplaceID,
    );

    foreach my $i ( keys %args ) {
        $request{$i} = $args{$i};
    }

    return $self->query(\%request);
}

# build ASIN list from EANs
sub GetMatchingProductForId {
    my $self = shift;
    my (%args) = @_;

    unless ( defined $args{'EANs'} ){
        Error('XMLFileError', {
            'Reason'  =>, 'no incomming EANs',
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }

    unless ( defined $args{'IdType'} ){
        Error('XMLFileError', {
            'Reason'  =>, 'no incomming IdType',
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }

    my $EANList = $args{'EANs'};
    my $aMatchedProducts = [];

    while (scalar(@$EANList)) {
        my @EANParts = splice(@$EANList,0,5);
        my ($ahFoundProducts, $Error) = $self->_doGetMatchingProducts({'EANs'=> \@EANParts, 'IdType' => $args{'IdType'} });
        # handling for "request throtling" errors:
        # we can do 20 requests at once with 5 EANs per call
        # after this call wait 1 second so we can do 5 calls (each with 5 EANs) at once afterwards
        if ($Error) {
            if ($Error->{'Code'} eq 'RequestThrottled') {
               push @$EANList, @EANParts;
               sleep 1;
            } else {
                Error($Error->{'Code'}, $Error);
            }
        } else {
            push @$aMatchedProducts, @$ahFoundProducts ;
        }
    }

    return $aMatchedProducts;
}

sub _doGetMatchingProducts {
    my $self = shift;
    my ($hargs) = @_;

    my $increment = 1;
    my %IdList;

    foreach my $EAN (@{$hargs->{'EANs'}}){
        $EAN =~ s/-//g;
        $IdList{'IdList.Id.'.$increment++} = $EAN;
    }
    my %request = (
        %IdList,
        'IdType'            => $hargs->{'IdType'},
        'Action'            => 'GetMatchingProductForId',
        'SellerId'          => $self->sellerID,
        'MWSAuthToken'      => $self->mwsAuthToken,
        'SignatureVersion'  => '2',
        'Version'           => '2011-10-01',
        'MarketplaceId'     => $self->marketplaceID,
    );

    delete $hargs->{'EANs'};

    foreach my $i ( keys %$hargs ) {
        $request{$i} = $hargs->{$i};
    }

    my $Response = $self->query(\%request);
    my $Decoded = decode_utf8($Response->content);

    # decode xml into perl structure
    my $xml = XML::Simple->new();
    my $NoXML = eval { $xml->XMLin( $Decoded ); };
    Warning( GetError() ) if ExistsError();

    my $Error =  undef;
    if(exists $NoXML->{'Error'}) {
        $Error = $NoXML->{'Error'};
    }

    my $ahProductsForIDs = $NoXML->{'GetMatchingProductForIdResult'};

    return ($ahProductsForIDs, $Error) if ref $ahProductsForIDs eq 'ARRAY';
    return ([$ahProductsForIDs], $Error);
 }

sub marketplaceID {
    return shift->{'marketplaceID'};
}

sub sellerID {
    return shift->{'sellerID'};
}

sub mwsAuthToken {
    return shift->{'mwsAuthToken'};
}

sub loggerKey {
    return 'RemoteMWSProduct';
}

1;
