#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# description  validate and get the basic informations from Shippinglabel
#----------------------------------------------------------------------------------------
# example      my $ProductData = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $ShippingLabel );
#               if ($ProductData->isComplete();){print "all is fine!\n";}
#               or:
#               $ProductData->loadAttributes();
#               my $TotalWeight = $ProductData->{'TotalWeight'};
#========================================================================================
package DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use feature 'switch';
use Date::Format;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::DHLIntraship::API::Constants qw (
    $EKP_LENGHT
    $PRODUCTCODE_LENGHT
    %MAPPING_PRODUCTCODE_PRODUCTNAME
    DELIVERY_DATE_IN_THE_PAST_WARNING_STRING
    DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
);
use DE_EPAGES::DHLIntraship::API::Tools;

use strict;


sub new {
    my $class = shift;
    my ( $ShippingLabel, $ServiceObject )  = @_;
    my $self  = bless {
        'ShippingLabel'  => $ShippingLabel,
        'DHLTools' => DE_EPAGES::DHLIntraship::API::Tools->new(),
        'WarningStates'  => [],
    }, $class;
    return $self;
}

#========================================================================================
# function     isComplete
# state        public
#----------------------------------------------------------------------------------------
# syntax       isComplete( );
#----------------------------------------------------------------------------------------
# description  checks if all mandatory Atrributes are ok
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | if all Attributes are fine | boolean
#========================================================================================
sub isComplete{
    my $self = shift;
    $self->loadAttributes();
    return 0 unless (scalar @{$self->{'WarningStates'}} == 0);
    return 1;
}

#========================================================================================
# function     loadAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       loadAttributes( );
#----------------------------------------------------------------------------------------
# description  load all Attributes to the class object
#========================================================================================
sub loadAttributes {
    my $self = shift;
    $self->_loadAlias();
    $self->_loadEstimatedDeliveryDay();
    $self->_loadShippingFulfillmentMethod();
    $self->_loadProductCode();
    return;
}

#========================================================================================
# function     getStatesAsWarning
# state        public
#----------------------------------------------------------------------------------------
# syntax       getStatesAsWarning( $Input1 );
#----------------------------------------------------------------------------------------
# description  Get the States and create epages warnings from this
#----------------------------------------------------------------------------------------
# return       $Warnings | epages warnings | refarray
#========================================================================================
sub getStatesAsWarning{
    my $self = shift;
    my $aWarnings =  BlockWarnings (
        sub {
             foreach my $hWarningState ( @{$self->{'WarningStates'}} ){
                 my %Warning = ();
                 $Warning{'attribute'} = $hWarningState->{'attribute'};

                if ($hWarningState->{'reason'} eq DELIVERY_DATE_IN_THE_PAST_WARNING_STRING) {
                    $Warning{'reason'} = DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD;
                } else {
                    $Warning{'reason'} = MANDATORY_WARNING_KEYWORD;
                }
                eval {Warning('ShippingLabelValidationWarning', \%Warning)};
                if ( ExistsError() ) {
                    my $Error = GetError();
                    LogError($Error) if ( $Error->code =~ /Native/ );
                }
            }
        },
    );
    return $aWarnings;
}

#========================================================================================
# function     _isValidAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isValidAttribute( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Check if in the current instance of this class is an Attribute with the
#               AttributeName. If not it will store the Name for warnings.
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | String
# return       $ReturnVariable | true if the Attribute is valid otherwise false | boolean
#========================================================================================
sub _isValidAttribute {
    my $self = shift;
    my ( $AttributeName ) = @_;
    unless ( isValid( $self->{$AttributeName} ) ) {
        push( @{$self->{'WarningStates'}}, { 'attribute' => $AttributeName, 'reason' => MANDATORY_WARNING_STRING } );
        return 0;
    }
    return 1;
}

#========================================================================================
# function     _loadAlias
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadAlias( );
#----------------------------------------------------------------------------------------
# description  load the Alias from Shippinglabel
#========================================================================================
sub _loadAlias {
    my $self = shift;
    $self->{'Alias'} = $self->{'ShippingLabel'}->alias();
    $self->_isValidAttribute('Alias');
    return;
}

#========================================================================================
# function     _loadEstimatedDeliveryDay
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadEstimatedDeliveryDay( );
#----------------------------------------------------------------------------------------
# description  load the Estimated Delivery Day from Shippinglabel
#========================================================================================
sub _loadEstimatedDeliveryDay {
    my $self = shift;
    my $EstimatedDeliveryDate = $self->{'ShippingLabel'}->get('EstimatedDeliveryDate');
    if ( defined $EstimatedDeliveryDate ) {
        $self->{'EstimatedDeliveryDate'} = time2str('%Y-%m-%d', $EstimatedDeliveryDate->epoch);
        $self->_isValidDeliveryDate('EstimatedDeliveryDate', $EstimatedDeliveryDate);
    }
    return;
}

#========================================================================================
# function     _isValidDeliveryDate
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isValidDeliveryDate( $AttributeName, $EstimatedDeliveryDate );
#----------------------------------------------------------------------------------------
# description  Check if provided delivery date is in the past (not valid).
#----------------------------------------------------------------------------------------
# input        $AttributeName | Attribute name | string
# input        $EstimatedDeliveryDate | date object | object
# return       $ReturnVariable | true if the estimated delivery date is not in the past | boolean
#========================================================================================
sub _isValidDeliveryDate {
    my $self = shift;
    my ($AttributeName, $EstimatedDeliveryDate ) = @_;

    return 1 unless (isValid($EstimatedDeliveryDate));

    my $Tools = DE_EPAGES::DHLIntraship::API::Tools->new();

    if ($Tools->IsDateInThePast($EstimatedDeliveryDate)) {
        push( @{$self->{'WarningStates'}}, { 'attribute' => $AttributeName, 'reason' => DELIVERY_DATE_IN_THE_PAST_WARNING_STRING } );
        return 0;
    }
    return 1;
}

#========================================================================================
# function     _loadShippingFulfillmentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadShippingFulfillmentMethod( );
#----------------------------------------------------------------------------------------
# description  load the shipping fulfillment method from Shippinglabel
#========================================================================================
sub _loadShippingFulfillmentMethod {
    my $self = shift;
    $self->{'ShippingFulfillmentMethod'} = $self->{'ShippingLabel'}->get('ShippingFulfillmentMethod');
    $self->_isValidAttribute('ShippingFulfillmentMethod');
    return;
}

#========================================================================================
# function     _loadProductCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadProductCode( );
#----------------------------------------------------------------------------------------
# description  Get all DHL services active in the Shipping Label
#========================================================================================
sub _loadProductCode {
    my $self = shift;
    Error( 'MissingParameter', {
            'Parameter' => 'ShippingFulfillmentMethod',
            'Message'   => 'ShippingFulfillmentMethod is not defined in ShippingLabel'
    } ) unless $self->{'ShippingFulfillmentMethod'};
    my $Order = $self->{'ShippingLabel'}->get('Order');
    my $Address = ($Order->get('ShippingAddress')) ? $Order->get('ShippingAddress') : $Order->get('BillingAddress');
    my $ProductCodeID = substr (
                            $self->{'DHLTools'}->GetCustomerNumber($self->{'ShippingFulfillmentMethod'}, $Address),
                            $EKP_LENGHT,
                            $PRODUCTCODE_LENGHT
                        );
    $self->{'ProductCode'} = $MAPPING_PRODUCTCODE_PRODUCTNAME{$ProductCodeID};
    $self->_isValidAttribute('ProductCode');
    return;
}
1;