#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabel_Validation
# state        public
#----------------------------------------------------------------------------------------
# description  class for unit test realted to API::ShippingLabelAttributes::ShippingLabel.pm
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabel_Validation;
use base DE_EPAGES::Test::API::UnitTest::Test;

use strict;

use DE_EPAGES::DHLIntraship::API::Constants qw (
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
    TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD
    TOTAL_WEIGHT_TOO_HIGH_WARNING_STRING
    TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD
    TOTAL_WEIGHT_TOO_LOW_WARNING_STRING
);
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify::Matcher qw ( String );
use Test::Mockify::Sut;
use Test::More;
use Test::MockObject;
use Test::Exception;

use constant MILIMETER_TO_CENTIMETER_FACTOR => 10;
use constant LINE_ITEMS_SUM => 39.99;
use constant PARCEL_DEPTH_IN_MILIMETERS => 255;
use constant PARCEL_LENGTH_IN_MILIMETERS => 125;
use constant PARCEL_WIDTH_IN_MILIMETERS => 90;
use constant TOTAL_WEIGHT => 1.5;
use constant TOTAL_WEIGHT_TOO_LOW => 0.05;
use constant TOTAL_WEIGHT_TOO_HIGH => 81.5;


#sub Main {
sub test {
    my $self = shift;
    $self->test_Products_no_warnings();
    $self->test_Products_weight_too_high_warning();
    $self->test_Products_weight_too_low_warning();
    $self->test_Products_line_item_sum_warning();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_Products_no_warnings {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject();

    my $SUT = $self->_createSUT_Products($ShippingLabelMock);

    ok($SUT->isComplete(), "$subTestName - products completion is OK.");

    my $aWarningStates = $SUT->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 0, "$subTestName - WarningStates array is empty");
    is($SUT->{'TotalWeight'}, TOTAL_WEIGHT ,"$subTestName - products total weight found into validator.");
    is($SUT->{'ParcelDepth'}, PARCEL_DEPTH_IN_MILIMETERS / MILIMETER_TO_CENTIMETER_FACTOR ,"$subTestName - parcel depth found into validator.");
    is($SUT->{'ParcelLength'}, PARCEL_LENGTH_IN_MILIMETERS / MILIMETER_TO_CENTIMETER_FACTOR ,"$subTestName - parcel length found into validator.");
    is($SUT->{'ParcelWidth'}, PARCEL_WIDTH_IN_MILIMETERS / MILIMETER_TO_CENTIMETER_FACTOR ,"$subTestName - parcel width found into validator.");
    is($SUT->{'LineItemsSum'}, LINE_ITEMS_SUM ,"$subTestName - line item sum found into validator.");
}

sub test_Products_weight_too_high_warning {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'TotalWeight' => TOTAL_WEIGHT_TOO_HIGH }
    );

    my $SUT = $self->_createSUT_Products($ShippingLabelMock);

    ok(!$SUT->isComplete(), "$subTestName - products completion is NOT OK.");

    my $aWarningStates = $SUT->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 1, "$subTestName - WarningStates array HAS ONE warning state.");

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(), 'ProductsValidationWarning', "$subTestName - it is a products warning.");

    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'TotalWeight', 'reason' => TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD },
            "$subTestName - states as warning method returns total weight too high keyword."
    );

    is_deeply($SUT->{'WarningStates'}->[0],
            { 'attribute' => 'TotalWeight', 'reason' => TOTAL_WEIGHT_TOO_HIGH_WARNING_STRING },
            "$subTestName - warning state holds total weight too high string message."
    );
}

sub test_Products_weight_too_low_warning {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'TotalWeight' => TOTAL_WEIGHT_TOO_LOW }
    );

    my $SUT = $self->_createSUT_Products($ShippingLabelMock);

    ok(!$SUT->isComplete(), "$subTestName - products completion is NOT OK.");

    my $aWarningStates = $SUT->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 1, "$subTestName - WarningStates array HAS ONE warning state.");

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(), 'ProductsValidationWarning', "$subTestName - it is a products warning.");

    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'TotalWeight', 'reason' => TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD },
            "$subTestName - states as warning method returns total weight too low keyword."
    );

    is_deeply($SUT->{'WarningStates'}->[0],
            { 'attribute' => 'TotalWeight', 'reason' => TOTAL_WEIGHT_TOO_LOW_WARNING_STRING },
            "$subTestName - warning state holds total weight too low string message."
    );
}

sub test_Products_line_item_sum_warning {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'LineItemsSum' => '' }
    );

    my $SUT = $self->_createSUT_Products($ShippingLabelMock);

    ok(!$SUT->isComplete(), "$subTestName - products completion is NOT OK.");

    my $aWarningStates = $SUT->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 1, "$subTestName - WarningStates array HAS ONE warning state.");

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(), 'ProductsValidationWarning', "$subTestName - it is a products warning.");

    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'LineItemsSum', 'reason' => MANDATORY_WARNING_KEYWORD },
            "$subTestName - states as warning method returns line items sum mandatory keyword."
    );

    is_deeply($SUT->{'WarningStates'}->[0],
            { 'attribute' => 'LineItemsSum', 'reason' => MANDATORY_WARNING_STRING },
            "$subTestName - warning state holds line items sum mandatory string message."
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createSUT_Products {
    my $self = shift;
    my ($ShippingLabelMock) = @_;

    my $aParams = [ $ShippingLabelMock ];
    my $Mockify = Test::Mockify::Sut->new('DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products', $aParams);
    $Mockify->mockImported('DE_EPAGES::Object::API::Factory', 'LoadObjectByPath')
        ->when(
            String('/Units/metre/centimetre')
        )
        ->thenReturn($self->_createMock_UnitOfMeasurement(MILIMETER_TO_CENTIMETER_FACTOR));

    return $Mockify->getMockObject();
}

sub _createMock_ShippingLabelObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    $hAtributeParams //= {};
    my $hAtributes = {
        'LineItemContainer' => $hAtributeParams->{'LineItemContainer'} // $self->_createMock_LineItemContainerObject($hAtributeParams),
        'ParcelDepth' => $hAtributeParams->{'ParcelDepth'} // PARCEL_DEPTH_IN_MILIMETERS,
        'ParcelLength' => $hAtributeParams->{'ParcelLength'} // PARCEL_LENGTH_IN_MILIMETERS,
        'ParcelWidth' => $hAtributeParams->{'ParcelWidth'} // PARCEL_WIDTH_IN_MILIMETERS,
        'TotalWeight' => $hAtributeParams->{'TotalWeight'} // TOTAL_WEIGHT
    };
    my $MockifyAddressObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $hAtributes
    );

    return $MockifyAddressObject->getMockObject();
}

sub _createMock_LineItemContainerObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    my $hAtributes = {
        'LineItemsSum' => $hAtributeParams->{'LineItemsSum'} // LINE_ITEMS_SUM
    };
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder',
            $hAtributes
    );

    return $MockifyObjectObject->getMockObject();
}

sub _createMock_UnitOfMeasurement {
    my $self = shift;
    my ($Factor) = @_;
    
    my $hAtributes = {
        'Factor' => $Factor
    };
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
            $hAtributes
    );

    return $MockifyObjectObject->getMockObject();
}


__PACKAGE__->runScript();