#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabel_Validation
# state        public
#----------------------------------------------------------------------------------------
# description  class for unit test realted to API::ShippingLabelAttributes::ShippingLabel.pm
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabel_Validation;
use base DE_EPAGES::Test::API::UnitTest::Test;

use Date::Format qw ( time2str );
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );

use strict;


use DE_EPAGES::DHLIntraship::API::Constants qw (
    $GERMANY_COUNTRY_ID
    DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD
    DELIVERY_DATE_IN_THE_PAST_WARNING_STRING
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::MockObject;
use Test::Exception;


#sub Main {
sub test {
    my $self = shift;
    $self->test_ShippingLabel_no_warnings();
    $self->test_ShippingLabel_estimated_delivery_date_warning();
    $self->test_ShippingLabel_product_code_warning();
    $self->test_ShippingLabel_no_shipping_address_error();
    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_ShippingLabel_no_warnings {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject();

    my $ShippingLabelValidation = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new($ShippingLabelMock);

    ok($ShippingLabelValidation->isComplete(), "$subTestName - shipping label completion is OK.");
    is($ShippingLabelValidation->{'Alias'},'MyTestShippingLabelAlias' ,"$subTestName - shipping label alias found into validator.");
    my $ExpectedEstimatedDeliveryDateString = time2str('%Y-%m-%d', $self->_dateTime_2_days_in_the_future()->epoch());
    is($ShippingLabelValidation->{'EstimatedDeliveryDate'},$ExpectedEstimatedDeliveryDateString ,"$subTestName - shipping label estimated delivery date found into validator.");

    my $aWarningStates = $ShippingLabelValidation->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 0, "$subTestName - WarningStates array is empty");
}

sub test_ShippingLabel_estimated_delivery_date_warning {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $EstimatedDeliveryDateInThePast = GetCurrentDateTime()->subtract('days' => 2);
    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'EstimatedDeliveryDate' => $EstimatedDeliveryDateInThePast }
    );

    my $ShippingLabelValidation = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new($ShippingLabelMock);

    ok(!$ShippingLabelValidation->isComplete(), "$subTestName - shipping label completion is NOT OK.");

    my $aWarningStates = $ShippingLabelValidation->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 1, "$subTestName - WarningStates array HAS ONE warning state.");

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(), 'ShippingLabelValidationWarning', "$subTestName - it is a shipping label warning.");

    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'EstimatedDeliveryDate', 'reason' => DELIVERY_DATE_IN_THE_PAST_WARNING_KEYWORD },
            "$subTestName - states as warning method returns delivery date in the past keyword."
    );

    is_deeply($ShippingLabelValidation->{'WarningStates'}->[0],
            { 'attribute' => 'EstimatedDeliveryDate', 'reason' => DELIVERY_DATE_IN_THE_PAST_WARNING_STRING },
            "$subTestName - warning state holds delivery date in the past string message."
    );
}

sub test_ShippingLabel_product_code_warning {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'EKP' => 'ThisIsAnInvalidEKP' }
    );

    my $ShippingLabelValidation = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new($ShippingLabelMock);

    ok(!$ShippingLabelValidation->isComplete(), "$subTestName - shipping label completion is NOT OK.");

    my $aWarningStates = $ShippingLabelValidation->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 1, "$subTestName - WarningStates array HAS ONE warning state.");

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(), 'ShippingLabelValidationWarning', "$subTestName - it is a shipping label warning.");

    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'ProductCode', 'reason' => MANDATORY_WARNING_KEYWORD },
            "$subTestName - states as warning method returns not set keyword (product code)."
    );

    is_deeply($ShippingLabelValidation->{'WarningStates'}->[0],
            { 'attribute' => 'ProductCode', 'reason' => MANDATORY_WARNING_STRING },
            "$subTestName - warning state holds mandatory attribute not set (product code)."
    );
}


sub test_ShippingLabel_no_shipping_address_error {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShippingLabelMock = $self->_createMock_ShippingLabelObject(
            { 'ShippingFulfillmentMethod' => '' }
    );

    my $ShippingLabelValidation = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new($ShippingLabelMock);

    throws_ok(
           sub{$ShippingLabelValidation->isComplete()}
           ,qr/Message=ShippingFulfillmentMethod is/,
           "$subTestName - error when no shipping fulfillment method found."
    );

    my $aWarningStates = $ShippingLabelValidation->getStatesAsWarning();
    is(scalar(@{$aWarningStates}), 1, "$subTestName - warning states counter is one.");
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createMock_ShippingLabelObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    my $DefaultEstimatedDeliveryDate = $self->_dateTime_2_days_in_the_future();
    $hAtributeParams //= {};
    my $hAtributes = {
        'Alias' => $hAtributeParams->{'Alias'} // 'MyTestShippingLabelAlias',
        'EstimatedDeliveryDate' => $hAtributeParams->{'EstimatedDeliveryDate'} // $DefaultEstimatedDeliveryDate,
        'Order' => $self->_createMock_OrderObject($hAtributeParams),
        'ShippingFulfillmentMethod' => $hAtributeParams->{'ShippingFulfillmentMethod'} // $self->_createMock_ShippingFulfillmentMethodObject($hAtributeParams)
    };
    my $MockifyAddressObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $hAtributes
    );

    return $MockifyAddressObject->getMockObject();
}

sub _createMock_OrderObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    my $hAtributes = {
        'ShippingAddress' => $hAtributeParams->{'ShippingAddress'} // $self->_createMock_AddressObject($hAtributeParams)
    };
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder',
            $hAtributes
    );

    return $MockifyObjectObject->getMockObject();
}

sub _createMock_ShippingFulfillmentMethodObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    $hAtributeParams //= {};
    my $hAtributes = {
        'EKP' => $hAtributeParams->{'EKP'} // '2222222222',
        'TeilnehmerNational' => '01',
        'TeilnehmerWeltpaket' => '01'
    };
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship',
            $hAtributes
    );

    return $MockifyObjectObject->getMockObject();
}

sub _createMock_AddressObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    $hAtributeParams //= {};
    my $hAtributes = {
        'CountryID' => $hAtributeParams->{'CountryID'} // $GERMANY_COUNTRY_ID
    };
    my $MockifyAddressObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes);

    return $MockifyAddressObject->getMockObject();
}

sub _dateTime_2_days_in_the_future {
    return GetCurrentDateTime()->add('days' => 2);
}

__PACKAGE__->runScript();