#========================================================================================
# package      DE_EPAGES::Order::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the shop functions of order.
#========================================================================================
package DE_EPAGES::Order::API::Shop;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetLatestOrderCreationDate
    GetNextOrderNo
    ProcessOrderDocumentTextAttributes
    ProcessOrderDocumentTextAttributesByLanguageID
    ShouldSplitShippingAndPaymentTax
);

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Shop::API::Shop qw( GetNextNo );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::Order::API::Constants qw (NO_SPLIT_SHIPPING_TAX NO_SPLIT_PAYMENT_TAX);
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw( ProcessPage );
use DE_EPAGES::Core::API::Log qw (SetLogLevel);

#========================================================================================
# function     GetNextOrderNo
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $OrderNo = GetNextOrderNo($Shop);
# syntax       my $OrderNo = GetNextOrderNo($Shop, $Object);
#----------------------------------------------------------------------------------------
# description  Returns the next order alias for this shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $Object | object; used if OrderNo shall contain this objects alias
#                       | DE_EPAGES::Object::API::Object::Object
# return       $OrderNo | alias string for order | string
#========================================================================================
sub GetNextOrderNo($;$) {
    my ($Shop, $Object) = @_;
    return GetNextNo($Shop, 'CustomerOrder', $Object);
}

#========================================================================================
# function     ProcessOrderDocumentTextAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessOrderDocumentTextAttributes($Shop, $PageTypeAlias, $Update);
#----------------------------------------------------------------------------------------
# description  set shop attributes for order documents by the given PageType alias
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $PageTypeAlias | PageType to be processed | string
# input        $Update | attributes are always updated | boolean
#========================================================================================
sub ProcessOrderDocumentTextAttributes {
    my ($Shop, $PageTypeAlias, $Update) = @_;

    my $hVars = $Shop->tleHash;
    my @ShopLanguages = map {$_->{'LanguageID'}} @{$Shop->get('Languages')};

    my $PageType = LoadPageTypeByAlias($PageTypeAlias);
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$Shop);

    my $Text;
    SetLogLevel('WARN', sub {
        $Text = ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($Shop->get('LanguageID')), $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    foreach my $Text2Key ('OrderDocumentsInvoiceText2','OrderDocumentsPackingSlipText2','OrderDocumentsCreditNoteText2'){
        foreach my $ShopLanguageID (@ShopLanguages){
            if($Update or not defined $Shop->get($Text2Key, $ShopLanguageID) ){
                $Shop->set({ $Text2Key => $Text }, $ShopLanguageID );
            }
        }
    }

}

#========================================================================================
# function     ProcessOrderDocumentTextAttributesByLanguageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessOrderDocumentTextAttributesByLanguageID($Shop, $PageTypeAlias, $Update, $LanguageID);
#----------------------------------------------------------------------------------------
# description  set shop attributes for order documents by the given PageType alias and language id
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $PageTypeAlias | PageType to be processed | string
# input        $Update | attributes are always updated | boolean
# input        $LanguageID | language id | string
#========================================================================================
sub ProcessOrderDocumentTextAttributesByLanguageID {
    my ($Shop, $PageTypeAlias, $Update, $LanguageID) = @_;

    my $hVars = $Shop->tleHash;
    my $PageType = LoadPageTypeByAlias($PageTypeAlias);
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$Shop);

    my $Text;
    SetLogLevel('WARN', sub {
        $Text = ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($LanguageID), $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    foreach my $Text2Key ('OrderDocumentsInvoiceText2','OrderDocumentsPackingSlipText2','OrderDocumentsCreditNoteText2'){
        if($Update or not defined $Shop->get($Text2Key, $LanguageID) ){
            $Shop->set({ $Text2Key => $Text }, $LanguageID );
        }
    }
}


#========================================================================================
# function     GetLatestOrderCreationDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LatestOrderCreationDate = GetLatestOrderCreationDate($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the datetime of the latest created order of the given shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $LatestOrderCreationDate | latest oder creation date | datettime
#========================================================================================
sub GetLatestOrderCreationDate {
    my ($Shop) = @_;

    my $dbi = GetCurrentDBHandle();
    my $sql = <<'END_SQL';
SELECT MAX(oo.creationdate) FROM customerorder AS o
INNER JOIN object AS oo ON oo.objectid = o.orderid AND oo.siteid=o.shopid
WHERE o.shopid = ?
END_SQL
    my $aaRows = $dbi->execute($sql, [$Shop->id], ['DateTime']);
    return $aaRows->[0]->[0];
}

#========================================================================================
# function     ShouldSplitShippingAndPaymentTax
# state        public
#----------------------------------------------------------------------------------------
# syntax       my ($SplitShippingTaxes, $SplitPaymentTaxes) = @{ ShouldSplitShippingAndPaymentTax($Shop) };
#----------------------------------------------------------------------------------------
# description  Returns, if the taxes for shipping and payment should be splitted
#               for the given shop (address)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $SplitShippingTaxes | split shipping taxes | boolean
# return       $SplitPaymentTaxes  | split payment taxes  | boolean
#========================================================================================
sub ShouldSplitShippingAndPaymentTax {
    my ($Shop) = @_;
    my $ShopAddress = $Shop->get('Address');
    my $ShopCountry = defined $ShopAddress ? $ShopAddress->get('Country') : undef;

    # EPG-18579 Pro-rata tax for delivery costs doesn't apply for France
    # split taxes on shipping if we are not in a country of the list NO_SPLIT_SHIPPING_TAX
    # shipping in @NO_SPLIT_SHIPPING_TAX do have the normal tax rate also if the product tax rate is reduced
    # that means, if we have a book with e.g. 7% tax we have to use 19% tax for the shipping method
    my $SplitShippingTaxes = (defined $ShopCountry and ElementOf($ShopCountry->{'Code2'},
        NO_SPLIT_SHIPPING_TAX)) ? 0 : 1;

    # EPG-20021 Pro-rata VAT for payment costs
    # in some countries like "FR GB ES" taxes for payment are not splitted (defined in: NO_SPLIT_PAYMENT_TAX)
    my $SplitPaymentTaxes = (defined $ShopCountry and ElementOf($ShopCountry->{'Code2'},
        NO_SPLIT_PAYMENT_TAX)) ? 0 : 1;

    return [$SplitShippingTaxes, $SplitPaymentTaxes];
}

1;
