#========================================================================================
# package      DE_EPAGES::ProductPortal::API::Object::PortalConfig
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for PortalConfig.
#========================================================================================
package DE_EPAGES::ProductPortal::API::Object::PortalConfig;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf GetCurrentDateTime unique UsePackage);
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObject);
##no critic (RequireCorrectImports)
use DE_EPAGES::ProductPortal::API::XML::ProductPortalImportHandler;
use DE_EPAGES::ProductPortal::API::XML::ProductPortalExportDriver;
##use critic
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use DE_EPAGES::ProductPortal::API::Table::PortalProductMap qw (
    DeletePortalProductMap
    ExistsPortalProductMap
    GetPKeyPortalProductMapsByPortalConfigID
    InsertPortalProductMap
);

sub ErrorDefinition {
    return {
        'TaxModelMismatch'               => 'tax model of portal does not match tax model of shop',
        'DOWNLOAD_PRODUCTS_NOT_ASSIGNED' => 'download products can not be assigned to this portal config',
        'PRODUCT_ALREADY_ASSIGNED'       => 'product #ProductAlias is already assigned to this portal config.',
    }
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PortalConfig->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PortalConfig' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PortalConfig->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig = DE_EPAGES::ProductPortal::API::Object::PortalConfig->insert($hInfo);
# example      my $Class = LoadClassByAlias('PortalConfig');
#               my $PortalConfig = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPortalConfig' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PortalConfig | new object | object
# hook         OBJ_InsertPortalConfig | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalConfigID - Portal Config identifier - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertPortalConfig', {'Object' => $self, 'PortalConfigID' => $self->id });

    delete $hInfo->{$_}
        foreach ( qw (  Shop
                        Portal
                        PortalSite
                        PortalMerchantID
                        Add2Basket
                        ExportAllProducts
                        ExportAllVariations
                        ShippingMethod
                        ImageAttribute
                        DescriptionAttribute
                        ShortDescriptionAttribute
                        LastExportDate
                        LastUploadDate
                        ExportInterval
                        FTPLogin
                        FTPPassword
                        FTPFileName )
                    ) ;

    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePortalConfig' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePortalConfig | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalConfigID - Portal Config identifier - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePortalConfig', {'Object' => $self, 'PortalConfigID' => $self->id });
    DeletePortalProductMap($_) foreach @{GetPKeyPortalProductMapsByPortalConfigID($self->id)};
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePortalConfig' and 'OBJ_AfterUpdatePortalConfig'.
#----------------------------------------------------------------------------------------
# input        $hValues | values to set | ref.hash
# input        $LanguageID | language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePortalConfig | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalConfigID - Portal Config identifier - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePortalConfig | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PortalConfigID - Portal Config identifier - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdatePortalConfig', {'Object'=>$self, 'Values'=> $hValues, 'PortalConfigID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePortalConfig',  {'Object'=>$self, 'Values'=> $hValues, 'PortalConfigID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     addProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->addProducts($aProducts);
#----------------------------------------------------------------------------------------
# description  Add products to portal, if a product is an super product all sub products
#               will be added.
#----------------------------------------------------------------------------------------
# input        $aProducts | products | ref.array.object
#========================================================================================
sub addProducts {
    my $self = shift;
    my ($aProducts) = @_;

    my @Products;
    if (!$self->get('PortalSite')->get('ExportsDownloadProducts')) {
        @Products = grep { !$_->get('IsDownloadProduct') } @$aProducts;
        if (scalar(@Products) < scalar(@$aProducts)) {
            my @DownloadProducts = grep { $_->get('IsDownloadProduct') } @$aProducts;
            Warning('DOWNLOAD_PRODUCTS_NOT_ASSIGNED', {'ProductIDs' => [map {$_->id} @DownloadProducts], 'ProductAliases' => [map {$_->alias} @DownloadProducts]});
        }
    }
    else {
        @Products = @$aProducts;
    }

    # prepare check if the product is already assigned to the portal
    my @ProductIDs = map { $_->id } @Products;
    my $ConfigID = $self->id;
    foreach my $ProductID (unique(@ProductIDs)) {
        if (ExistsPortalProductMap($ConfigID, $ProductID)) {
            Warning('PRODUCT_ALREADY_ASSIGNED', {'ProductID' => $ProductID, 'ProductAlias' => LoadObject($ProductID)->alias});
        }
        else {
            InsertPortalProductMap({
                'PortalConfigID' => $ConfigID,
                'ProductID'      => $ProductID,
            });
        }
    }
    return;
}

#========================================================================================
# function     existsProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->existsProduct($Product);
#----------------------------------------------------------------------------------------
# description  Checks if product exists on product portal
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# return       $existsProduct | product is assigned to this portal | boolean
#========================================================================================
sub existsProduct {
    my $self = shift;
    my ($Product) = @_;

    return ExistsPortalProductMap($self->id, $Product->id);
}

#========================================================================================
# function     removeProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->removeProducts($aProducts);
#----------------------------------------------------------------------------------------
# description  Removes products from portal.
#----------------------------------------------------------------------------------------
# input        $aProducts | products | ref.array.object
#========================================================================================
sub removeProducts {
    my $self    = shift;
    my ($aProducts) = @_;

    my $ConfigID = $self->id;
    foreach my $Product (grep { ExistsPortalProductMap($ConfigID, $_->id) } @$aProducts) {
        DeletePortalProductMap($ConfigID, $Product->id);
    }
    return;
}

#========================================================================================
# function     removeAllProducts
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->removeAllProducts;
#----------------------------------------------------------------------------------------
# description  Removes all products from portal.
#========================================================================================
sub removeAllProducts {
    my $self    = shift;

    DeletePortalProductMap($_->{'PortalConfigID'}, $_->{'ProductID'}) foreach @{GetPKeyPortalProductMapsByPortalConfigID($self->id)};
    return;
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->exportDriver;
#----------------------------------------------------------------------------------------
# description  Retrieves export driver.
#----------------------------------------------------------------------------------------
# return       $ExportDriver | name of export driver | string
#========================================================================================
sub exportDriver {
    return shift->get('Portal')->get('ExportDriver');
}

#========================================================================================
# function     exportPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->exportPortal($FileName);
#----------------------------------------------------------------------------------------
# description  Exports product data which are relevant to Portal.
#----------------------------------------------------------------------------------------
# input        $FileName | name of export file including path | string
# input        $NoMonitor | do not show a monitor page | boolean
#========================================================================================
sub exportPortal {
    my $self = shift;
    my ($FileName, $NoMonitor) = @_;

    my $Package = $self->exportDriver;

    my $ShopTaxModel = $self->getSite()->get('TaxModel');
    my $PortalTaxModel = $self->get('PortalSite')->get('TaxModel');
    Error('TaxModelMismatch') if $ShopTaxModel != $PortalTaxModel;

    UsePackage($Package);
    my $Driver = $Package->new;
    if ($Package =~ /CSV/) {
        $Driver->exportCSV($FileName, undef, undef, $self, $NoMonitor, $self);
    } else {
        $Driver->exportXML($FileName, undef, $self, $NoMonitor );
    }
    $self->set({'LastExportDate' => GetCurrentDateTime()});
    return;
}

#========================================================================================
# function     exportXMLElements
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->exportXMLElements($Driver, $allAttributes);
#----------------------------------------------------------------------------------------
# description  Exports product mapping.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $allAttributes | all attributes (not used, see SUPER) | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;

    if ($Driver->exportLevel == $self->class->attribute('Products')->get('ExportLevel')) {
        $Driver->PortalProductMapsByPortalConfig($Driver);
    }
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $Object = $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes );

    # delete product map if no reference (start of config)
    if (!$Handler->isReference) {
        DeletePortalProductMap($_->{'PortalConfigID'}, $_->{'ProductID'}) foreach @{GetPKeyPortalProductMapsByPortalConfigID($Object->id)};
    }
    return $Object;
}

#========================================================================================
# function     uploadPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalConfig->uploadPortal($FileName);
#----------------------------------------------------------------------------------------
# description  This implementation does nothing and returns always -1.
#----------------------------------------------------------------------------------------
# input        $PathFileName | path file name of file to upload (optional) | string
# return       $ret | 0 - failure, 1 - success, -1 - no upload necessary | int
#========================================================================================
sub uploadPortal {
    my $self = shift;
    my ($FileName) = @_;

    return -1; # -1 means no upload necessary
}

#========================================================================================
# function     shippingMethodForProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ShippingMethod = $PortalConfig->shippingMethodForProduct($Product);
#----------------------------------------------------------------------------------------
# description  Find the shipping method to be used for shipping cost calculation. If no
#               shipping method is assigned to the portal config, returns undef. Returns
#               the assigned shipping method if it is suitable for the product. Otherwise
#               returns the shop's default shipping method if it can be used for the product
#               and the portal site's country. Returns any other shipping method under the
#               same conditions. Returns undef if no suitable shipping method found.
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# return       $ShippingMethod | shipping method to be used for shipping cost calculation or undef if none found | object
#========================================================================================
sub shippingMethodForProduct {
    my $self = shift;
    my ($Product) = @_;

    # 1. shipping method of the portal - do not check regions here as the merchant gets a warning in MBO
    my $PortalConfigShippingMethod = $self->get('ShippingMethod');
    return undef unless defined $PortalConfigShippingMethod;
    return $PortalConfigShippingMethod unless defined $Product;

    my $aProductShippingMethods = $Product->getWithParent('ShippingMethods');
    return $PortalConfigShippingMethod if (scalar(@$aProductShippingMethods) == 0); # all shipping methods available for the product
    return $PortalConfigShippingMethod if (ElementOf($PortalConfigShippingMethod, @$aProductShippingMethods));

    # 2. default shipping method if suitable for product and country
    my $aAvailableShippingMethods = $self->get('AvailableShippingMethods');
    my $DefaultShippingMethod = undef;
    foreach my $sm (@$aAvailableShippingMethods) {
        next if ($sm->id == $PortalConfigShippingMethod->id);
        if ($sm->get('IsDefault')) {
            $DefaultShippingMethod = $sm;
            last;
        }
    }
    return $DefaultShippingMethod if ($self->_isShippingMethodUsable($DefaultShippingMethod, $aProductShippingMethods));

    # 3. any other shipping method
    foreach my $sm (@$aAvailableShippingMethods) {
        next if ($sm->id == $PortalConfigShippingMethod->id);
        next if (defined($DefaultShippingMethod)  &&  $sm->id == $DefaultShippingMethod->id);
        return $sm if ($self->_isShippingMethodUsable($sm, $aProductShippingMethods));
    }
    return undef;
}

#========================================================================================
# function     _isShippingMethodUsable
# state        private
#----------------------------------------------------------------------------------------
# syntax       if (_isShippingMethodUsable($ShippingMethod, $aProductShippingMethods)) {...
#----------------------------------------------------------------------------------------
# description  checks if the shipping method is element of the product's shipping methods
#               and if it can be used in the portal site's country.
#----------------------------------------------------------------------------------------
# input        $ShippingMethod | shipping method to check | object
# input        $aProductShippingMethods | list of product's shipping methods | ref.array.object
# return       $isShippingMethodUsable | shipping method can be used | boolean
#========================================================================================
sub _isShippingMethodUsable {
    my $self = shift;
    my ($ShippingMethod, $aProductShippingMethods) = @_;

    return 0 unless defined $ShippingMethod;
    return 0 unless ElementOf($ShippingMethod, @$aProductShippingMethods);

    if (defined(my $Region = $ShippingMethod->get('Region'))) {
        my $PortalConfigCountryID = $self->get('PortalSite')->get('Country')->{'CountryID'};
        foreach my $Country (@{$Region->get('Countries')}) {
            return 1 if ($Country->{'CountryID'} == $PortalConfigCountryID);
        }
        return 0;
    }
    else {
        return 1;
    }
}

#========================================================================================
# function     allProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $PortalConfig->allProducts;
#----------------------------------------------------------------------------------------
# description  Returns the product identifiers to be exported. If the option
#               "ExportAllVariation" is selected all SubProducts of a MasterProduct
#               are exported, else only default SubProducts are exported.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub allProducts {
    my $self = shift;

    my $ahProducts = LoadClassByAlias('Product')->packageObject->dal->get('ByShopID', [$self->getSite()->id]);
    #  EPG-20574 Invisible products should not be exported
       $ahProducts = [grep { $_->{'IsVisible'} } @$ahProducts];

    # remove download products if portal does not export them
    if (!$self->get('PortalSite')->get('ExportsDownloadProducts')) {
        $ahProducts = [grep { !$_->{'IsDownloadProduct'} } @$ahProducts];
    }


    if ($self->get('ExportAllVariations')) {
        return [map { $_->{'ProductID'} } grep { !$_->{'IsAppointment'}  &&  !defined($_->{'DefaultSubProductID'}) } @$ahProducts];
    }
    else {
        my @ProductIDs = ();
        foreach my $hInfo (@$ahProducts) {
            next if $hInfo->{'IsAppointment'};
            next if defined($hInfo->{'SuperProductID'});
            push(@ProductIDs, defined($hInfo->{'DefaultSubProductID'})  ?  $hInfo->{'DefaultSubProductID'} : $hInfo->{'ProductID'});
        }
        return \@ProductIDs;
    }
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $PortalConfig->items;
#----------------------------------------------------------------------------------------
# description  Returns all product identifier of portal config
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub items {
    my $self = shift;

    if ($self->get('ExportAllProducts')) {
        # export all products
        return $self->allProducts();
    }

    my @Products = ();
    my @AssignedProductIDs = map { $_->{'ProductID'} } @{GetPKeyPortalProductMapsByPortalConfigID($self->id)};

    if ($self->get('ExportAllVariations')) {
        # export SubProducts only
        foreach my $ProductID (@AssignedProductIDs) {
            my $Product = LoadObject($ProductID);
            if ($Product->get('DefaultSubProduct')) {
                my $aSubProducts = $Product->get('SubProducts');
                push(@Products, $_) foreach grep { $_->get('IsVisible') } @$aSubProducts;
            }
            else {
                push(@Products, $Product) if ($Product->get('IsVisible'));
            }
        }
    }
    else {
        # if SuperProduct then export DefaultSubProduct only
        foreach my $ProductID (@AssignedProductIDs) {
            my $Product = LoadObject($ProductID);
            my $DefaultSubProduct = $Product->get('DefaultSubProduct');
            if (defined($DefaultSubProduct) && $DefaultSubProduct->get('IsVisible')) {
                push(@Products, $DefaultSubProduct);
            }
            else {
                push(@Products, $Product) if ($Product->get('IsVisible'));
            }
        }
    }

    if (!$self->get('PortalSite')->get('ExportsDownloadProducts')) {
        @Products = grep { !$_->get('IsDownloadProduct') } @Products;
    }

    # The base products are replaced by their subproducts, so we could have repeated items if the merchant
    # mixes base and subproducts.
    # To avoid repeated items we apply the unique function ...
    @Products = unique( map { $_->id } @Products ) ;

    return \@Products ;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ProductPortal::API::Object::PortalConfig->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $ProductClass = LoadClassByAlias('Product');
    my %Info = (
        'Add2Basket'           => 0,
        'ExportAllProducts'    => 0,
        'ExportAllVariations'  => 0,
        'ImageAttribute'       => $ProductClass->attribute('ImageSmall'),
        'DescriptionAttribute' => $ProductClass->attribute('Description'),
        'ExportInterval'       => 0,
    );
    if (defined $hInfo->{'Parent'}) {
        my $Shop = $hInfo->{'Parent'}->getSite();
        $Info{'Shop'} = $Shop;
        my $aShippingMethods = $Shop->get('ShippingMethods');
        my ($DefaultShippingMethod) = grep { $_->get('IsDefault') } @$aShippingMethods;
        $Info{'ShippingMethod'} = $DefaultShippingMethod;
    }
    $Info{'Portal'} = $hInfo->{'PortalSite'}->get('Portal') if defined $hInfo->{'PortalSite'};

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        %Info
    };
}

#========================================================================================
# function     isConfigured
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsConfigured = $self->isConfigured( );
#----------------------------------------------------------------------------------------
# description  Returns boolean value informing about portal config being configured.
#               As a basic implementation, it checks if portal configuration has assigned products.
#               Overloading methods may look for additional setup like entered credentials.
#----------------------------------------------------------------------------------------
# return       $IsConfigured | portal config is configured or not | boolean
#========================================================================================
sub isConfigured {
    my $self = shift;

    my $ExportAllProducts = $self->get('ExportAllProducts');
    if ($ExportAllProducts) {
        return 1;
    }

    my $PortalConfigProductsCounter = $self->get('CountProducts');
    if ($PortalConfigProductsCounter > 0) {
        return 1;
    }

    return 0;
}

1;
