#========================================================================================
# package      DE_EPAGES::ProductRecommendation::API::Recommendation
# state        public
# description  Sends the product.recommendation e-mail.
#========================================================================================
package DE_EPAGES::ProductRecommendation::API::Recommendation;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    SendProductRecommendationMail
);

use Email::Address;
use DE_EPAGES::Core::API::File qw ( ExistsFile );
use DE_EPAGES::Core::API::TestParameter qw( TestHash );
use DE_EPAGES::Core::API::Url qw( Relative2Absolute ); ##no critic qw(DE_EPAGES::Style::RequireCorrectImports)  # Relative2Absolute is used in RegEx
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );


#========================================================================================
# function     SendProductRecommendationMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendProductRecommendationMail( $Product, $hParams );
# example      SendProductRecommendationMail( $Product, {
#                   'Receiver'          => $Receiver,
#                   'ReceiverEmail'     => $ReceiverEmail,
#                   'Sender'            => $Sender,
#                   'SenderEmail'       => $SenderEmail,
#                   'Greeting'          => $Greeting,
#                   'CurrencyID'        => $CurrencyID,
#                   'LocaleID'          => $LocaleID,
#                   'Copy'              => $Copy,
#               });
#----------------------------------------------------------------------------------------
# description  Sends a product.recommendation e-mail to $Receiver <$ReceiverEmail>
#               on behalf of $Sender <$SenderEmail>.
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $hParams | options
#                 <ul>
#                   <li>Receiver - recipient name - string</li>
#                   <li>ReceiverEmail - recipient emain</li>
#                   <li>Sender - sender name - string</li>
#                   <li>SenderEmail - sender email - string</li>
#                   <li>CurrencyID - (optional, default: shop currency) currency id, e.g. 'EUR' - string</li>
#                   <li>LocaleID - (optional, default: shop locale) locale id, e.g. 'de_DE' - string</li>
#                   <li>Copy - (optional) send a copy to the sender - boolean</li>
#                 </ul> | ref.hash
#========================================================================================
sub SendProductRecommendationMail {
    my ( $Product, $hParams ) = @_;
    TestHash('hParams', $hParams, {
        'Receiver' => 'String',
        'ReceiverEmail' => 'String;NotNull',
        'Sender' => 'String',
        'SenderEmail' => 'String;NotNull',
        'Greeting' => 'String',
        'CurrencyID' => 'String',
        'LocaleID' => 'String',
    });

    my $Shop = $Product->getSite();

    my $MailType  = GetShopMailType( $Shop, 'ProductRecommendation' );

    my %Vars = %{$hParams};
    $Vars{'Shop'} = $Shop->tleHash;
    $Vars{'SessionCurrencyID'} = $hParams->{'CurrencyID'};
    $Vars{'Email'} = $hParams->{'SenderEmail'};

    my $LocaleID = $hParams->{'LocaleID'} || $Shop->get('LocaleID');
    my $LanguageID = GetInfoLocale($LocaleID)->{LanguageID};
    my %RegionalParams = (
        Locale => $LocaleID,
        Language => GetCodeByLanguageID( $LanguageID ),
    );

    my $ReplyAddr = Email::Address->new($hParams->{'Sender'}, $hParams->{'SenderEmail'})->format;
    my @Receipients = (
        Email::Address->new($hParams->{'Receiver'}, $hParams->{'ReceiverEmail'})->format
    );
    push @Receipients, $ReplyAddr if $hParams->{'Copy'};

    foreach my $Receipient (@Receipients) {
        # FROM address is taken from shop mailtype
        my %MailParams = (
            'TO'     => $Receipient,
            'REPLY'  => $ReplyAddr,
            'SkipBadRecipients' => 0,
            'HTMLAllowed' => 1,

        );

        my $Mail = $MailType->getMail( $Product, \%MailParams, \%RegionalParams, \%Vars );

        # attach inline picture
        my $ImageFilename = $Product->getWithParent('ImageSmall');
        if( defined $ImageFilename ) {
            my $ImageFile = $Product->get('PublicPath').'/'.$ImageFilename;
            if( ExistsFile( $ImageFile ) ) {
                $Vars{'Image'} = 1;
                $Vars{'ImageIsLink'} =  $ImageFilename =~ /\/\// ? 1:0;
                $Mail->addHTMLAttachment([ {
                    'filename'  => $ImageFile,
                    'includeid' => '__ePagesPRC__',
                    'type'      => 'image/jpg'
                } ]);
            }
        }

        # send mail
        $MailType->sendMail( $Mail );
    }

    return;
}

1;
