#========================================================================================
# package      DE_EPAGES::ShippingTypes::UI::ShippingMethodLevels
# state        private
#----------------------------------------------------------------------------------------
# description  User interface for ShippingMethodPriceOfBasket,
#               ShippingMethodProductsInBasket and ShippingMethodWeightOfBasket
#========================================================================================
package DE_EPAGES::ShippingTypes::UI::ShippingMethodLevels;
use base DE_EPAGES::Shipping::UI::ShippingMethod;

use strict;
use DE_EPAGES::ShippingTypes::API::Constants;
use DE_EPAGES::Object::API::Factory                 qw( Transaction LoadClassByAlias );
use DE_EPAGES::Core::API::PerlTools                 qw( CopyStructure ElementOf unique );
use DE_EPAGES::Order::API::Shop                     qw ( ShouldSplitShippingAndPaymentTax );

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingMethod = $Servlet->object;

    my $Form = $Servlet->form;
    my $Image = $Form->value('Image');
    my $Alias = $Form->value('Alias');

    #--- save shipping logo and exit sub if it is only an image upload (EPG-20735)
    if($Image && ! $Alias){
        $ShippingMethod->set({'Image' => $Image});
        return;
    }

    my $Shop = $ShippingMethod->getSite();
    my $TaxModel = $Shop->get('InputTaxModel');
    my $hValues = $Form->form($ShippingMethod, 'SaveMultiLevel', 1);

    # Start EPG-28685: in the template there are two CurrencyID array field so one hast to be renamed
    # this hack renames the ValueCurrencyID because its needed for the money valitation
    $Form->{'Values'}{'CurrencyID'} = $Form->{'Values'}{'ValuesCurrencyID'};
    my $hFormData = $Form->formData( LoadClassByAlias('ContainerShippingMethod'), 'SaveShippingValues' );
    $hFormData->{'Loops'}{'ShippingValues'}{'CurrencyID'} = $hFormData->{'Loops'}{'ShippingValues'}{'ValuesCurrencyID'};
    $hFormData->{'Loops'}{'ShippingValues'}{'CurrencyID'}{'Name'} = 'CurrencyID';
    $hValues = {(%$hValues, %{$Form->checkValues( $ShippingMethod, $hFormData )})};
    # End EPG-28685

    # set ContainerSubTotalAttrName to LineItemsSubTotal if there is no selection box
    if (not defined $ShippingMethod->get('ContainerSubTotalAttrName') and $ShippingMethod->get('ShippingType')->get('Alias') eq 'PriceOfBasket') {
        $ShippingMethod->set({'ContainerSubTotalAttrName' => 'LineItemsSubTotal'});
    }

    # remember the entries and selections if save fails (more than one form changes)
    $Servlet->vars('ShippingValues', CopyStructure($hValues->{'ShippingValues'}));

    my %ShippingValuesByCurrency;
    my $Index = 0;
    foreach my $hValue (@{$hValues->{'ShippingValues'}}) {
        $Index += 1;
        my $CurrencyID = $hValue->{'ValuesCurrencyID'};
        $ShippingValuesByCurrency{$CurrencyID} = $hValue;

        my $MaximumWeight = $hValue->{'MaximumWeight'};
        if (defined($MaximumWeight)) {
            foreach my $hLevel (@{$hValues->{'ShippingLevels'}}) {
                #--- maximum weight undercuts a lower bound
                $Form->addFormError({
                        'Reason'    => 'MAXIMUM_CLASHES_LOWER_BOUND',
                        'LoopName'  => 'ShippingValues',
                        'Form'      => 'Save',
                        'Name'      => 'MaximumWeight',
                        'Index'     => $Index-1,
                }) if ($hLevel->{'LowerBound'} >= $MaximumWeight);
            }
        }

        delete $hValue->{'ValuesCurrencyID'};
        $ShippingValuesByCurrency{$CurrencyID} = $hValue;
    }

    my %LevelsByCurrency;
    foreach my $hLevel (@{$hValues->{'ShippingLevels'}}) {
        my $CurrencyID = $hLevel->{'CurrencyID'};
        $LevelsByCurrency{$CurrencyID} = [] unless exists $LevelsByCurrency{$CurrencyID};
        delete $hLevel->{'CurrencyID'};
        push @{$LevelsByCurrency{$CurrencyID}}, $hLevel;
    }

    #--- save new shipping level
    $Index = 0;
    foreach my $hShippingLevel (@{$hValues->{'NewShippingLevels'}}) {
        $Index += 1;
        my $CurrencyID = $hShippingLevel->{'NewCurrencyID'};
        my $LowerBound = $hShippingLevel->{'NewLowerBound'};
        my $Multiplier = $hShippingLevel->{'NewMultiplier'};
        my $BaseValue  = $hShippingLevel->{'NewBaseValue'};

        # lines with no input should be ignored
        next unless (defined $LowerBound or defined $BaseValue);

        #--- base value not defined
        $Form->addFormError({
                'Reason'    => 'INVALID_BASE_VALUE',
                'LoopName'  => 'ShippingLevels',
                'Form'      => 'Save',
                'Name'      => 'NewBaseValue',
                'Index'     => $Index-1,
        }) if (defined $LowerBound and not defined $BaseValue);
        #--- multiplier not defined
        $Form->addFormError({
                'Reason'    => 'MULTIPLIER_NOT_DEFINED',
                'LoopName'  => 'ShippingLevels',
                'Form'      => 'Save',
                'Name'      => 'NewMultiplier',
                'Index'     => $Index-1,
        }) if ((defined $BaseValue or defined $LowerBound) and not defined $Multiplier);
        #--- lower bound not defined
        $Form->addFormError({
                'Reason'    => 'LOWER_BOUND_NOT_DEFINED',
                'LoopName'  => 'ShippingLevels',
                'Form'      => 'Save',
                'Name'      => 'NewLowerBound',
                'Index'     => $Index-1,
        }) if (defined $BaseValue and not defined $LowerBound);

        $LevelsByCurrency{$CurrencyID} = [] unless exists $LevelsByCurrency{$CurrencyID};
        $ShippingValuesByCurrency{$CurrencyID} = {} unless exists $ShippingValuesByCurrency{$CurrencyID};
        my $MaximumWeight = $ShippingValuesByCurrency{$CurrencyID}->{'MaximumWeight'} // undef;
        my $ahLevels = $LevelsByCurrency{$CurrencyID};
        my ($hLevel) = grep { $_->{'LowerBound'} == $LowerBound } @$ahLevels;
        if (not defined $hLevel) {
            my $UpperBound = INFINITE_UPPER_BOUND;
            #--- lower bound negative
            $Form->addFormError({
                    'Reason'    => 'INVALID_LOWER_BOUND',
                    'LoopName'  => 'ShippingLevels',
                    'Form'      => 'Save',
                    'Name'      => 'NewLowerBound',
                    'Index'     => $Index-1,
            }) if ($LowerBound < 0);

            foreach my $hLevel (sort {$a->{'LowerBound'} <=> $b->{'LowerBound'}} @$ahLevels) {
                if (($hLevel->{'LowerBound'} < $LowerBound) and
                    ($LowerBound < $hLevel->{'UpperBound'})) {
                    #--- set upper bound according to clashing shipping level
                    $UpperBound = $hLevel->{'UpperBound'};
                    #--- update existing shipping level
                    $hLevel->{'UpperBound'} = $LowerBound;
                }
            }

            if (defined($MaximumWeight)) {
                #--- lower bound exceeds maximum weight
                $Form->addFormError({
                        'Reason'    => 'LOWER_BOUND_CLASHES_MAXIMUM',
                        'LoopName'  => 'ShippingLevels',
                        'Form'      => 'Save',
                        'Name'      => 'NewLowerBound',
                        'Index'     => $Index-1,
                }) if ($LowerBound >= $MaximumWeight);
            }

            #--- save new shipping level
            push @$ahLevels, {
                'BaseValue'  => $BaseValue,
                'LowerBound' => $LowerBound,
                'UpperBound' => $UpperBound,
                'Multiplier' => $Multiplier,
            };
        } else {
            #--- lower bound of the new level clashes with the lower bound of an existing level
            $Form->addFormError({
                    'Reason'    => 'LOWER_BOUND_CLASH',
                    'LoopName'  => 'NewShippingLevels',
                    'Form'      => 'Save',
                    'Name'      => 'NewLowerBound',
                    'Index'     => $Index-1,
            });
        }
    }
    $Form->executeFormError();

    # set tax class before prices
    $self->SUPER::Save($Servlet);

    # save all levels in all currencies
    foreach my $CurrencyID (keys %LevelsByCurrency) {
        my $ahLevels = $LevelsByCurrency{$CurrencyID} || [];
        $ShippingMethod->setLevels($CurrencyID, $TaxModel, $ahLevels);
        my $hShippingValues = $ShippingValuesByCurrency{$CurrencyID} || {};
        $ShippingMethod->setShippingValues($CurrencyID, $TaxModel, $hShippingValues);
    }

    return;
}


sub DeleteLevels {
    my $self = shift;
    my ($Servlet) = @_;

    #--- delete listed objects until accept
    return if ! $Servlet->params('ListObjectID');

    my $ShippingMethod = $Servlet->object;
    my $Shop = $ShippingMethod->getSite();
    my $TaxModel = $Shop->get('InputTaxModel');

    my @ListObjectIDs = $Servlet->form->value('ListObjectID');
    my $ShippingLevelClass = LoadClassByAlias('ShippingLevel');

    my @ListObjects = map {$ShippingLevelClass->loadObject($_)} @ListObjectIDs;
    my @delCurrencyIDs;
    foreach my $object (@ListObjects) {
        push @delCurrencyIDs, $object->get('CurrencyID') if not ElementOf($object->get('CurrencyID'), @delCurrencyIDs);
    }
    my $ahLevels;
    foreach my $CurrencyID (@delCurrencyIDs) {
        my $ahlevels = $ShippingMethod->getLevels($CurrencyID, $TaxModel);
        foreach my $level (@$ahlevels){
            $level->{'CurrencyID'} = $CurrencyID;
            $level->{'ID'} = $level->{'ShippingLevelID'};

            push @$ahLevels,$level if ElementOf($level->{'ShippingLevelID'},@ListObjectIDs)
        }
    }

    #--- delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DeleteObjects',$ahLevels);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        #--- else reset delete
        $Servlet->vars('DELETE',0);
    }

    my @CurrencyIDs;
    Transaction( sub {
        foreach my $ListObject (@ListObjects) {
            push @CurrencyIDs, $ListObject->get('CurrencyID');
            $ListObject->delete;
        }
        my $ExistingShippingLevels = $ShippingMethod->levels;

        foreach my $CurrencyID (unique @CurrencyIDs) {
            my ($ShippingLevel) = grep { $_->get('CurrencyID') eq $CurrencyID } @$ExistingShippingLevels;
            if ($ShippingLevel) {
                my @ExistingShippingLevelsByCurrency = grep { $_->get('CurrencyID') eq $CurrencyID} sort {$a->get('LowerBound') <=> $b->get('LowerBound')} @$ExistingShippingLevels;
                #--- update lower bound of the first shipping level remaining in the list because the levels before are deleted, set zero
                my $FirstLevel = shift @ExistingShippingLevelsByCurrency;
                $FirstLevel->set({'LowerBound' => 0}) if ($FirstLevel->get('LowerBound') != 0);
                #--- update upper bound of the remaining shipping level(s)
                my $PreShippingLevel = $FirstLevel;
                foreach my $Level (@ExistingShippingLevelsByCurrency) {
                    if ($Level->get('LowerBound') !=  $PreShippingLevel->get('UpperBound')) {
                        $PreShippingLevel->set({'UpperBound' => $Level->get('LowerBound')});
                    }
                    $PreShippingLevel = $Level;
                }

                #--- update last shipping level in list, set upper bound infinite
                if (scalar @ExistingShippingLevelsByCurrency == 0) {
                    #--- first level is last level
                    $FirstLevel->set({'UpperBound' => INFINITE_UPPER_BOUND});
                } else {
                    my $LastLevel = pop @ExistingShippingLevelsByCurrency;
                    $LastLevel->set({'UpperBound' => INFINITE_UPPER_BOUND}) if defined $LastLevel;
                }
            }
        }
    } );

    # set shipping method invisible if shiping is incomplete
    my $IsActivated = $ShippingMethod->get('IsActivated');
    if ($IsActivated == 1) {
        my $aShopCurrencies = $Shop->get('Currencies');
        my @ActiveCurrencies = grep { $_->{'IsActivated'} == 1 } @$aShopCurrencies;
        my $hVars = { 'TaxModel'   => $Shop->get('TaxModel') };
        foreach my $ActiveCurrency (@ActiveCurrencies) {
            $hVars->{'CurrencyID'} = $ActiveCurrency->{'CurrencyID'};
            my $SettingsComplete = $ShippingMethod->usableAtStorefront($hVars);
            if ($SettingsComplete == 0) {
                $ShippingMethod->set({'IsActivated' => 0});
                $Servlet->vars('SetInvisible', 1);
            }
        }
    }

    return;
}

sub UnityViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite();
    my $SplitShippingTaxes = ShouldSplitShippingAndPaymentTax($Shop)->[0];

    # Only display tax class selection on settings page in case it is not splitted (UNITY-4811)
    $Servlet->vars('DisplayTaxClass', $SplitShippingTaxes ? 0 : 1);

    $self->View($Servlet);
}

1;
