#========================================================================================
# package      DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler;
# base         DE_EPAGES::CSVExportImport::API::CSV::ObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to import shop objects
# example      use DE_EPAGES::CSVExportImport::API::CSV::ObjectImportHandler;
#               DE_EPAGES::CSVExportImport::API::CSV::ObjectImportHandler->importCSV($FileName, $hFileFormat, $Shop);
#========================================================================================
package DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler;
use base qw ( DE_EPAGES::CSVExportImport::API::CSV::ObjectImportHandler Exporter );

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Shop::API::Constants qw( Text2TaxModel  );
use DE_EPAGES::Shop::API::Table::ShopCurrency qw( ExistsShopCurrency );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode ExistsLanguageByCode );
use DE_EPAGES::Object::API::Factory qw( LoadObject );
use DE_EPAGES::Core::API::File qw ( fsplit ExistsFile CopyFile IsFileNewer );
use DE_EPAGES::Core::API::Url qw ( StringIsURL );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#ERRORDEFINITION
# Be sure to add your warning in the cartridge's Constants.pm list!!!
sub ErrorDefinition {
    return {
        'VOID_FIELD'               => 'line #Line, column #ID: #Error',
        'MISSING_FIELD'            => 'CSV file (#FileName), line 1: no such column (#Value)',
        'WRONG_FORMAT'             => 'line 1, column #ID, CSV file (#FileName): void format (#Column)',
        'NO_MAND_VALUE'            => 'line #Line, column #Attribute: has empty value (but is mandatory).',
        'NO_MONEY_FORMAT'          => 'line 1, column #ID, CSV file (#FileName): no money format (#Column)',
        'NO_SHOP_CURRENCY'         => 'line 1, column #ID, CSV file (#FileName): currency is not in shop (#Column)',
        'NO_SHOP_LANG'             => 'line 1, column #ID, CSV file (#FileName): language is not in shop (#Column)',
        'NO_SHOP_TAXMODEL'         => 'line 1, column #ID, CSV file (#FileName): tax model is not in shop (#Column)',
        'REFERENCED_IMAGE_MISSING' => 'line #Line, column #AttributeName: referenced image not found',
        'UNKNOWN_OBJECT'           => 'line #Line, #Value is not an object',
        'FORBIDDEN_OBJECT'         => 'line #Line, #Value is not an object of type #Class',
        'REFERENCED_FILE_MISSING'  => 'line #Line, column #AttributeName: referenced file not found',
        'STRING_TOO_LONG'          => 'line #Line: the value of the attribute "#Attribute" is too long (maximum length: #MaxLength Characters).',
    };
}

#ERRORDEFINITION

#========================================================================================
# function     analyzeHeaderLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hUsedAttributes = $self->analyzeHeaderLine();
#----------------------------------------------------------------------------------------
# description  Overwrites base method to previously add some specific Shop Items, like
#               locales, units and so on
#----------------------------------------------------------------------------------------
# return       $hUsedAttributes | properties of each column of CSV file
#               <li>$ColumnIndex  | integer | index of column
#                   <ul>
#                       <li>Attribute  - e.g. Class, Alias, Color
#                       <li>Class      - AttributeClass e.g. Jacket
#                       <li>Type       - Attribute type, e.g. LocalizedString
#                       <li>IsObject   - 0 or 1, set to 1 if Attribute IsObject
#                       <li>LanguageID - only set if Type =~ /Localized/
#                       <li>CurrencyID - only set if Type eq 'Price'
#                       <li>TaxModel   - 0 (==net) or 1, only set if Type eq 'Price'
#                       <li>LocaleID   - only set for DATETIME_TYPES, hash, e.g. 'en_US'
#                   </ul>
#               | ref.hash of hashes
#========================================================================================
sub analyzeHeaderLine {
    my $self = shift;
    my $Shop = $self->object;

    my %ShopLocales = ();
    map { $ShopLocales{ $_->get('LocaleID') } = $_ } @{ $Shop->get('Locales') };
    $self->pushID( 'ShopLocales', \%ShopLocales );

    my %ShopUnits = ();
    map { $ShopUnits{ $_->alias } = $_ } @{ $Shop->units() };
    $self->pushID( 'ShopUnits', \%ShopUnits );

    my %ShopTaxClasses = ();
    map { $ShopTaxClasses{ $_->alias } = $_ } @{ $Shop->get('TaxMatrix')->get('TaxClasses') };
    $self->pushID( 'ShopTaxClasses', \%ShopTaxClasses );

    return $self->SUPER::analyzeHeaderLine;
}

#========================================================================================
# function     getUsedAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getUsedAttribute($HeaderString, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  return attribute info, called by analyzeHeaderLine
#               throws Error 'UNKNOWN_ATTR' if $hHeader->{'AliasString'} isn't a key of $self->attributes
#               call function testHeaderColumn before return $hAttribute
#----------------------------------------------------------------------------------------
# input        $HeaderString | header string | string
# input        $ColumnIndex |  index of current column | integer
# return       $hAttribute | properties of column of CSV file with following keys
#                       <ul>
#                           <li>Attribute  | attribute object | object
#                           <li>Alias  | alias of the attribute | string
#                           <li>AliasString  | alias string of the column e.g Shoe/ShoeWeight/en | string
#                           <li>ClassAlias | alias of the attribute class e.g. Shoe | string
#                           <li>HeaderString  | header string of the column e.g 'Schuh/Gewicht/English [Shoe/ShoeWeight/en]' | string
#                           <li>IsBaseClassAttribute | 0 or 1, set to 1 if attribute is from base class | boolean
#                           <li>IsObject   | 0 or 1, set to 1 if Attribute IsObject | boolean
#                           <li>Type       | Attribute type, e.g. LocalizedString | String
#                           <li>LanguageID | only set if Type =~ /Localized/ | Integer
#                           <li>CurrencyID | only set if Type eq 'Price' | String
#                           <li>TaxModel   | 0 (==net) or 1, only set if Type eq 'Price' | Integer
#                       </ul> | ref.hash
#========================================================================================
sub getUsedAttribute {
    my $self = shift;
    my ( $HeaderString, $ColumnIndex) = @_;

    my $hHeader = $self->headerColumn($HeaderString, $ColumnIndex+1);

    my $hAttributes = $self->attributes;
    my $hAttribute = $hAttributes->{$hHeader->{'AliasString'}};

    # handle wrong columns for unity or epages6
    if(not defined $hAttribute) {

        # do not throw an error if we have an unity shop and an unknown row
        my $Shop = $self->object;
        if($Shop->get('IsUnityRegistered')){

            # Important! Here we add a handling to ignore missing attributes in the header line
            # we create a FAKE entry for the attribute (without a real attribute) to prevent errors on checking the headline
            # if we import the values line by line we ignore this attribute placeholder
            # 'IGNORE_COLUMN' is here used to identify columns to ignore

            $hAttribute->{'IGNORE_COLUMN'} = 1;
            $hAttribute->{'Alias'}      = 'Position';
            $hAttribute->{'ClassAlias'} = 'ProductClass';
            $hAttribute->{'AliasString'} = 'Position';

            $hAttribute->{'HeaderString'} = 'xxx [Position]';
            $hAttribute->{'Type'} = 'String';
            $hAttribute->{'IsObject'} = 0;
            $hAttribute->{'IsBaseClassAttribute'} = 1;
            $hHeader->{'HeaderParts'} = ['Position'];

        } else {
            Error('UNKNOWN_ATTR',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} });
        }
    }

    my $aHeaderParts = $hHeader->{'HeaderParts'};
    unshift @$aHeaderParts, $self->baseAttributeClassAlias if $hAttribute->{'IsBaseClassAttribute'} == 1;
    $self->testHeaderColumn($hAttribute, $hHeader, $ColumnIndex );
    return $hAttribute;
}

#========================================================================================
# function     modifyUsedAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->modifyUsedAttributes($hUsedAttributes, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  add keys to $hUsedAttributes->{$ColumnIndex}
#                 IsClassColumn = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Class' ? 1: 0
#                 IsAliasColumn = $hUsedAttributes->{$ColumnIndex}{'Alias'}  eq 'Alias' ? 1: 0
#                 IsParentColumn = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Parent' ? 1: 0
#               add keys to hUsedAttributes:
#                 ClassColumn with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsClassColumn'};
#                 AliasColumn with value=$ColumnIndex  if $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'};
#                 ParentColumn with value=$ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsParentColumn'};
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub modifyUsedAttributes {
    my $self = shift;
    my ( $hUsedAttributes, $ColumnIndex ) = @_;
    $hUsedAttributes->{$ColumnIndex}{'IsClassColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Class'  ? 1 : 0;
    $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'}  = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Alias'  ? 1 : 0;
    $hUsedAttributes->{$ColumnIndex}{'IsParentColumn'} = $hUsedAttributes->{$ColumnIndex}{'Alias'} eq 'Parent' ? 1 : 0;
    $hUsedAttributes->{'ClassColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsClassColumn'};
    $hUsedAttributes->{'AliasColumn'}  = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsAliasColumn'};
    $hUsedAttributes->{'ParentColumn'} = $ColumnIndex if $hUsedAttributes->{$ColumnIndex}{'IsParentColumn'};
    return;
}

#========================================================================================
# function     testMissingFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testMissingFields($hUsedAttributes);
#----------------------------------------------------------------------------------------
# description  throws error 'MISSING_FIELD', if field Alias is missing
#----------------------------------------------------------------------------------------
# input        $hUsedAttributes | used attributes | ref.hash
#========================================================================================
sub testMissingFields {
    my $self = shift;
    my ($hUsedAttributes) = @_;
    if ( !exists $hUsedAttributes->{'AliasColumn'} ) {
        Error( 'MISSING_FIELD' => { 'Value' => 'Alias', 'FileName' => $self->{'FileName'} } );
    }
}

#========================================================================================
# function     testHeaderColumn
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->testHeaderColumn($hAttribute, $hHeader, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  test $hHeader,
#               throws Errors NO_MONEY_FORMAT, NO_SHOP_CURRENCY, NO_SHOP_TAXMODEL, NO_SHOP_LANG, WRONG_FORMAT
#----------------------------------------------------------------------------------------
# input        $hAttribute | used attribute | ref.hash
# input        $hHeader    | current header info | ref.hash
# input        $ColumnIndex | current column | integer
#========================================================================================
sub testHeaderColumn {
    my $self = shift;
    my ( $hAttribute, $hHeader, $ColumnIndex ) = @_;

    my $aHeaderParts = $hHeader->{'HeaderParts'};
    my $HeaderString = $hHeader->{'HeaderString'};

    my $Type = $hAttribute->{'Type'};
    if ( $Type eq 'Price' ) {
        my $TaxModel;
        eval { $TaxModel = Text2TaxModel( $aHeaderParts->[3] ); };
        if ( ExistsError() or $TaxModel eq '' ) {
            Error( 'NO_MONEY_FORMAT',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
        if ( not ExistsShopCurrency( $self->object->id, $aHeaderParts->[2] ) ) {
            Error( 'NO_SHOP_CURRENCY',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
        if ( $TaxModel != $self->object->get('InputTaxModel') ) {
            Error( 'NO_SHOP_TAXMODEL',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
    }
    elsif ( $Type =~ /^Localized(String|File)$/ ) {
        if ( scalar(@$aHeaderParts) != 3 ) {
            Error( 'NO_SHOP_LANG',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
        elsif ( not ExistsLanguageByCode( $aHeaderParts->[2] ) ) {
            Error( 'NO_SHOP_LANG',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
        my $LangID = GetPKeyLanguageByCode( $aHeaderParts->[2] );
        if ( not $self->object->existsLanguage($LangID) ) {
            Error( 'NO_SHOP_LANG',
                { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
        }
    }
    elsif ( scalar(@$aHeaderParts) != 2 ) {
        Error( 'WRONG_FORMAT',
            { 'ID' => $ColumnIndex + 1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} } );
    }
    return;
}

#========================================================================================
# function     importObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importObject;
#----------------------------------------------------------------------------------------
# description  calls $hAttributeValues = $self->analyzeAttributeLine,
#               return if (not defined $hAttributeValues or $hAttributeValues->{'DoNotImport'})
#               if the object exists, it should be at $hAttributeValues->{'Object'}
#               then a $Object->set($hAttributeValues->{'DirectAttributes'}) is called
#               else $Class must exists $hAttributeValues->{'DirectAttributes'}{'Class'}
#               then calls $Class->insertObject($hAttributeValues->{'DirectAttributes'});
#               then calls foreach language id keys %{ $hAttributeValues->{'LocalizedAttributes'}} :
#               my $hInfo = $hAttributeValues->{'LocalizedAttributes'}{$LanguageID};
#               $Object->set($hInfo, $LanguageID);
#               calls function $self->importSpecialData($Object, $hAttributeValues); to give the possibility
#               to save more data
#========================================================================================
sub importObject {
    my $self = shift;

    my $hAttributeValues = $self->analyzeAttributeLine;
    return if ( not defined $hAttributeValues or $hAttributeValues->{'DoNotImport'} );

    my $Object            = $hAttributeValues->{'Object'};
    my $hDirectAttributes = $hAttributeValues->{'DirectAttributes'};
    my $hLocalizedAttributes = \%{ $hAttributeValues->{'LocalizedAttributes'} };
    my $hPriceAttributes = $hAttributeValues->{'PriceAttributes'};

    my $hOldProductAttributes;
    my $hOldLocalizedAttributes;

    my $Class = $hDirectAttributes->{'Class'};

    my $hHookParams = {
        'Class' => $Class,
        'Object' => $Object,
        'DirectAttributes' => $hDirectAttributes,
        'LocalizedAttributes' => $hLocalizedAttributes,
        'PriceAttributes' => $hPriceAttributes,
        'LineNumber' => $self->lineNumber
    };
    TriggerHook('OBJ_BeforeCSVImportObject', $hHookParams);
    return if $hHookParams->{'DoNotImport'};

    delete $hDirectAttributes->{'Class'};
    if ( defined $Object ) {

        # store former attributes
        foreach my $Attribute ( keys %$hDirectAttributes ) {
            $hOldProductAttributes->{$Attribute} = $Object->class->existsAttribute($Attribute) ? $Object->get($Attribute) : undef;
        }

        # change class before setting the other attributes
        $Object->set( { 'Class' => $Class } ) if $Class->id != $Object->class->id;
        $Object->set($hDirectAttributes);
    }
    else {
        eval { $Object = $Class->insertObject($hDirectAttributes); };
        if ( ExistsError() ) {
            my $Error     = GetError();
            my $ErrorCode = $Error->code;
            if ( $ErrorCode =~ /FeatureLimitExceeded/ || $ErrorCode =~ /TooManyLevelsOfCategories/ ) {
                $Error->addVars( {
                    Line => $self->lineNumber,
                    Alias => $hDirectAttributes->{'Alias'},
                    Parent => $hDirectAttributes->{'Parent'}
                } );
                Warning($Error);
                return;
            }
            else {
                Error($Error);
            }
        }
    }

    # update localized attributes
    foreach my $LanguageID ( keys %$hLocalizedAttributes ) {
        my $hInfo = $hLocalizedAttributes->{$LanguageID};

        # store former localized attributes
        foreach my $Attribute ( keys %$hInfo ) {
            $hOldLocalizedAttributes->{$LanguageID}{$Attribute} = $Object->class->existsAttribute($Attribute) ? $Object->get($Attribute, $LanguageID) : undef;
        }

        $Object->set( $hInfo, $LanguageID );
    }

    $self->importSpecialData( $Object, $hAttributeValues );

    # EPG-9416 : update object after creation/csv modify
    $self->modifyObject( $Object, $hOldProductAttributes, $hDirectAttributes, $hOldLocalizedAttributes, $hLocalizedAttributes );

    return;
}

#========================================================================================
# function     importSpecialData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importSpecialData($Object, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  possibility to import special object data, called by importObject
#               only dummy function here
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hAttributeValues    | attribute values | ref.hash
#========================================================================================
sub importSpecialData {
    my $self = shift;
    my ( $Object, $hAttributeValues ) = @_;
    return;
}

#========================================================================================
# function     modifyObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->modifyObject($Object, $hOldProductAttributes, $hDirectAttributes, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  possibility to modify the object after it's creation or modification by
#               the csv import. Called by importObject, only dummy function here.
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hOldProductAttributes | attributes of the object before the import | ref.hash
# input        $hDirectAttributes     | direct attribute values | ref.hash
#========================================================================================
sub modifyObject {
    my $self = shift;
    my ( $Object, $hOldProductAttributes, $hDirectAttributes ) = @_;
    return;
}

#========================================================================================
# function     cleanupAttribute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $String = $self->cleanupAttribute($String);
#----------------------------------------------------------------------------------------
# description  Strips leading slashes and whitespaces from the given string
#               and replace backslashes by slashes
#----------------------------------------------------------------------------------------
# input        $String | any string, most likely a file path | string
# return       $String | updated string without backslashes and leading slashes | string
#========================================================================================
sub cleanupAttribute {
    my $self = shift;
    my ($String) = @_;
    return $String if not $String or $String eq '';

    $String =~ s/^[\.\\\/\s]*//x;
    # replace (Windows) backslashes by slashes
    $String =~ s/\\/\//g;
    return $String;
}


#========================================================================================
# function     resetFlagsAndWarnings
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->resetFlagsAndWarnings();
#----------------------------------------------------------------------------------------
# description  reset the gloabl warnings counter.
#========================================================================================
sub resetFlagsAndWarnings {
    return;
}

#========================================================================================
# function     findImageFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->findImageFile($MediaGallery, $NewImageLarge);
# example      my ($NewImageLargePath, $NewImageLargeFileName,
#                   $NewImageLargeExtension, $MGNewImageLargeAbs)
#                    = $self->findImageFile($MediaGallery, $NewImageLarge);
#----------------------------------------------------------------------------------------
# description  Tries to find the given image file from the CSV import Attribute inside
#               the mediagallery and returns the fsplit filename parts as well as the
#               absolute path to the file in the mediagallery.
#----------------------------------------------------------------------------------------
# input        $MediaGallery | mediagallery object | object
# input        $CSVImageFile | file name / path given in the CSV Attribute | string
# return       ($CSVImagePath, $CSVImageFileName, $CSVImageExtension, $MGSCVImageAbs) |
#               list of the fsplit parts of the $CSVImageFile and the absolute path
#               to the file in the mediagallery | list
#========================================================================================
sub findImageFile {
    my $self = shift;
    my ( $MediaGallery, $CSVImageFile ) = @_;

    my ( $CSVImagePath, $CSVImageFileName, $CSVImageExtension ) = fsplit($CSVImageFile);
    my $MGSCVImageAbs = $MediaGallery->absolutePath($CSVImageFile);

    # search file in main mediagallery dir if it does not exists there
    if ( !ExistsFile($MGSCVImageAbs) ) {
        $MGSCVImageAbs = $MediaGallery->absolutePath( $CSVImageFileName . $CSVImageExtension );
        $MGSCVImageAbs = undef if !ExistsFile($MGSCVImageAbs);
    }

    return ( $CSVImagePath, $CSVImageFileName, $CSVImageExtension, $MGSCVImageAbs );
}

#========================================================================================
# function     fixObjectFileAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fixObjectFileAttributes($Object, \@AttributeNames, $LanguageID);
# example      $self->fixObjectFileAttributes($Product,
#                   ['ImageLarge', 'ImageMedium', 'ImageSmall', 'ImageHotDeal'],1);
#----------------------------------------------------------------------------------------
# description  Removes any paths from the image file name attributes of the given object.
#----------------------------------------------------------------------------------------
# input        $Object | Object | object
# input        $aAttributes | Array of the attribute names to fix | ref.array
# input        $LanguageID | LanguageID | string
#========================================================================================
sub fixObjectFileAttributes {
    my $self = shift;
    my ( $Object, $aAttributes, $LanguageID ) = @_;

    my $hCurrentAttributes = $Object->get($aAttributes,$LanguageID);

    foreach my $Attribute ( keys %$hCurrentAttributes ) {
        my $AttributeValue = $hCurrentAttributes->{$Attribute};
        if ( $hCurrentAttributes && $AttributeValue && !StringIsURL($AttributeValue) ) {
            my ( undef, $Name, $Ext ) = fsplit($AttributeValue);
            $hCurrentAttributes->{$Attribute} = $Name . $Ext;
        }
    }
    $Object->set($hCurrentAttributes,$LanguageID);
}

#========================================================================================
# function     processMediaGalleryImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hNewAttributes = $self->_processMediaGalleryImage(
#                   $hDirectAttributes,
#                   $SourceFileName,
#                   $TargetFileName
#               );
# example      $hDirectAttributes = $self->_processMediaGalleryImage(
#                   $hDirectAttributes,
#                   $MGNewImageLargeAbs,
#                   $Product->get('PublicPath') . '/' . $NewImageLargeFileName.$NewImageLargeExtension
#               );
#----------------------------------------------------------------------------------------
# description  Copies the product image from the given source and generates its
#               scaled variations. Updates the given attribute hash with the image names.
#----------------------------------------------------------------------------------------
# input        $Object | Category object which was imported | object
# input        $hDirectAttributes | Direct attributes | ref.hash
# input        $SourceFileName     | full file path and name of the source image | string
# input        $TargetFileName     | full file path and name for the target image | string
# return       $hDirectAttributes  | updated direct attribute values | ref.hash
# input        $ScaleImages | boolean value if automatic generation of other image sizes
#               should be applied.
# input        $hScaleDimension | Width and Height for the scaled image | ref.hash
# input        $DoNotUpdateAttributes | boolean value whether product attributes should be
#               updated | boolean
#========================================================================================
sub processMediaGalleryImage {
    my $self = shift;
    my ( $Object, $hDirectAttributes, $SourceFileName, $TargetFileName, $ScaleImages, $hScaleDimension,
        $DoNotUpdateAttributes )
    = @_;

    # copy to product folder
    CopyFile( $SourceFileName, $Object->get('PublicPath') . '/' . $TargetFileName, { 'create_directory' => 1 } );
}

#========================================================================================
# function     throwImageMissingWarning
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->throwImageMissingWarning($Object, $NewImageFile);
#----------------------------------------------------------------------------------------
# description  Check image file given using AttributeName and the hAttributeValues hash
#               is a URL string. Otherwise throw a warning that the image which is
#               referenced in the CSV file is missing.
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $NewImageFile  | image file name attribute values | ref.hash
# input        $AttributeName | Name of the attribute | string
#========================================================================================
sub throwImageMissingWarning {
    my $self = shift;
    my ( $Object, $NewImageFile, $AttributeName ) = @_;

    Warning(
        'REFERENCED_IMAGE_MISSING', {
            Line          => $self->lineNumber,
            Image         => $NewImageFile,
            ObjectAlias   => $Object->get('Alias'),
            ObjectID      => $Object->id,
            AttributeName => $AttributeName
        }
    );
    return;
}

#========================================================================================
# function     processImportedImageAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->processImportedImageAttribute(
#                   $Object, $hOldAttributes, $hDirectAttributes,
#                   $AttributeName, $ScaleImages
#               );
# example      $self->processImportedImageAttribute(
#                   $Category, $hOldAttributes, $hDirectAttributes, 'ImageSmall', 0
#               );
#----------------------------------------------------------------------------------------
# description  EPG-9416: Handler method to fetch object images from the MediaGallery.
#----------------------------------------------------------------------------------------
# input        $Object | Category object which was imported | object
# input        $hOldAttributes | attributes of the object before the import.
#               May be undef if the product is created new and not modified.
#               | ref.hash or undefined
# input        $hDirectAttributes | current Category's direct attribute values | ref.hash
# input        $AttributeName | Name of the attribute | string
# input        $ScaleImages | boolean value if automatic generation of other image sizes
#               should be applied. The parameter is just supplied to the
#               processMediaGalleryImage method, which implemention may differ in derived
#               classes.
# input        $hScaleDimension | Width and Height for the scaled image | ref.hash
#========================================================================================
sub processImportedImageAttribute {
    my $self = shift;
    my ( $Object, $hOldAttributes, $hDirectAttributes, $AttributeName, $ScaleImages, $hScaleDimension ) = @_;

    if ( !StringIsURL( $hDirectAttributes->{$AttributeName} ) ) {
        my $Site             = $Object->getSite();
        my $ObjectPublicPath = $Object->get('PublicPath');
        my $MediaGallery     = $Site->get('MediaGallery');

        # ********* CLEANUP ATTRIBUTE STRING *********
        my $NewImageLarge = $self->cleanupAttribute( $hDirectAttributes->{$AttributeName} );

        # ********* FIND NEW IMAGE FILE *********
        my ( undef, $NewImageLargeFileName, $NewImageLargeExtension, $MGNewImageLargeAbs )
        = $self->findImageFile( $MediaGallery, $NewImageLarge );

        # ********* CURRENT IMAGE FILE VALUE FOR THE OBJECT *********
        my $OldImageFile = defined $hOldAttributes ? $hOldAttributes->{$AttributeName} : undef;

        if ( defined $MGNewImageLargeAbs and ExistsFile($MGNewImageLargeAbs) ) {

            # ********* CASE 1: NEW IMAGE FILE EXISTS *********

            if ( not defined $OldImageFile ) {

                # ********* CASE 1.1: NO OLD IMAGE FILE *********
                $self->processMediaGalleryImage( $Object, $hDirectAttributes, $MGNewImageLargeAbs,
                    $NewImageLargeFileName . $NewImageLargeExtension,
                    $ScaleImages, $hScaleDimension );
                $hDirectAttributes->{$AttributeName} = $NewImageLargeFileName . $NewImageLargeExtension;
                $Object->set( { $AttributeName => $NewImageLargeFileName . $NewImageLargeExtension } );
            }
            else {
                if ( $OldImageFile eq $NewImageLargeFileName . $NewImageLargeExtension ) {

                    # ********* CASE 1.2: OLD IMAGE FILE WITH SAME NAME AS NEW IMAGE FILE *********
                    if ( not ExistsFile( $ObjectPublicPath . '/' . $OldImageFile )
                        or IsFileNewer( $ObjectPublicPath . '/' . $OldImageFile, $MGNewImageLargeAbs ) ) {

                        # ********* CASE 1.2.1: OLD FILE MISSING OR NEW FILE IS NEWER *********
                        # file is new, so continue processing
                        $self->processMediaGalleryImage( $Object, $hDirectAttributes, $MGNewImageLargeAbs,
                            $NewImageLargeFileName . $NewImageLargeExtension,
                            $ScaleImages, $hScaleDimension );
                    }
                    else {

                        # ********* CASE 1.2.2: NEW FILE IS NOT NEWER *********
                        # file is not newer, so keep the current one
                    }
                }
                else {

                    # ********* CASE 1.3: NEW IMAGE FILE HAS OTHER NAME AS OLD IMAGE FILE *********
                    # replace old image file
                    $self->processMediaGalleryImage( $Object, $hDirectAttributes, $MGNewImageLargeAbs,
                        $NewImageLargeFileName . $NewImageLargeExtension,
                        $ScaleImages, $hScaleDimension );
                    $hDirectAttributes->{$AttributeName} = $NewImageLargeFileName . $NewImageLargeExtension;
                    $Object->set( { $AttributeName => $NewImageLargeFileName . $NewImageLargeExtension } );
                }
            }

            # strip any paths from product image attributes (undo csv import changes)
            $self->fixObjectFileAttributes( $Object, [$AttributeName] );
        }
        else {

            # ********* CASE 2: NEW IMAGE FILE NOT FOUND *********
            if ( !ExistsFile( $ObjectPublicPath . '/' . $NewImageLargeFileName . $NewImageLargeExtension ) ) {

                # ********* CASE 2.1: NO OLD IMAGE FILE OR OLD IMAGE FILE DOES NOT EXIST *********
                $self->throwImageMissingWarning( $Object, $hDirectAttributes->{$AttributeName}, $AttributeName );
            }
        }
    }    # isURL

    return;
}

#========================================================================================
# function     loadShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->loadShopObject( $Shop, $Path, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  loads the object specified by object path and verifies that it exists,
#               belongs to the given parentobject and is an instance of a certain class.
#               Returns undef if path doesn't exist or object doesn't belong to the same shop.
#               Returns the object if all tests passed.
#----------------------------------------------------------------------------------------
# input        $ParentObject | parent object | shop object
# input        $Path | object path to load | string
# input        $ClassAlias | alias of a class (optional) | string
# return       $Object| object | object
#========================================================================================
sub loadShopObject {
    my $self = shift;
    my ($ParentObject, $Path, $ClassAlias) = @_;

    my $ObjectID = $ParentObject->findPath($Path);
    Error('UNKNOWN_OBJECT' => {
        'Value' => $Path,
        'Line' => $self->lineNumber(),
    }) unless defined $ObjectID;

    # get object
    my $ObjectByPath = LoadObject($ObjectID);
    # check if object belongs to the current shop
    Error('FORBIDDEN_OBJECT' => {
        'Value' => $Path,
        'Line' => $self->lineNumber(),
        'Class' => $ClassAlias,
    }) if (!$self->_checkShopObject($ParentObject->getSite(), $ObjectByPath, $ClassAlias));

    return $ObjectByPath;
}

#========================================================================================
# function     _checkShopObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_checkShopObject( $Shop, $Object, $ClassAlias )
#----------------------------------------------------------------------------------------
# description  Checks if an object belongs to a site (shop) and
#               is instance of a certain class. Returns 0 if not.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | shop object
# input        $Object | test object | object
# input        $ClassAlias | alias of a class (optional) | string
# return       $OK | true if check was ok | boolean
#========================================================================================
sub _checkShopObject {
    my $self = shift;
    my ($Shop, $Object, $ClassAlias) = @_;

    if ($Object->getSite()->id != $Shop->id) {
        return 0;
    }

    if ($ClassAlias and not $Object->instanceOf($ClassAlias)) {
        return 0;
    }

    return 1;
}

1;
