#========================================================================================
# package      DE_EPAGES::ShopNewsletter::Hooks::Shop
# state        public
# description  Hook module for newsletter.
#========================================================================================
package DE_EPAGES::ShopNewsletter::Hooks::Shop;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Newsletter::API::Constants qw( NEWSLETTER_SCHEDULED NEWSLETTER_EDITABLE );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Newsletter::UI::Site;

#========================================================================================
# function     OnAfterUpdateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  removes the scheduled date when a shop is closed by provider in order
#               to prevent deactivated shops from sending scheduled newsletters.
#               (Hook OBJ_AfterUpdateShop)
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object</li>
#                   <li>Values - attribute values - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    if( $hParams->{'Values'}->{'ClosedByProvider'} or $hParams->{'Values'}->{'ClosedByProviderTemporarily'}  ) {
        foreach my $Campaign ( @{$Shop->get('NewsletterCampaigns')} ) {
            foreach my $Newsletter ( @{$Campaign->get('Newsletters')} ) {
                if( NEWSLETTER_SCHEDULED == $Newsletter->get('Status') ) {
                    $Newsletter->set({
                        'Status' => NEWSLETTER_EDITABLE,
                        'ScheduleDate' => undef,
                    });
                }
            }
        }
    }
}

#========================================================================================
# function     OnJSONAdvancedLinks
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnJSONAdvancedLinks( $hParams );
#----------------------------------------------------------------------------------------
# description  adds a link to to advanced links section in the link picker inside the
#               MBO
#               (Hook OBJ_JSONAdvancedLinks)
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object</li>
#                   <li>Values - attribute values - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnJSONAdvancedLinks {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    if($Shop->featureMaxValue('Newsletter')){
        my $SitePath = $hParams->{'Object'}->pathString;
        push @{$hParams->{'Links'}},{
            Alias=>Translate('Newsletter',undef,'DE_EPAGES::ShopNewsletter'),
            Link=>"?ViewAction=ViewNewsletter&ObjectPath=$SitePath"
        };
    }
}

#========================================================================================
# function     OnBeforeSaveRegistration
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeSaveRegistration( $hParams );
#----------------------------------------------------------------------------------------
# description  saves newsletter - subscribed on registration. Sets Unsibscribed=2 and
#               SubscribedInCheckout=1.
#               (Hook UI_BeforeSaveRegistration)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeSaveRegistration {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    my $Form = $Servlet->form;
    my @NewsletterCampaignIDs = $Form->value('Campaign');
    return if 0 == scalar @NewsletterCampaignIDs;

    my $EMail = $Form->value('EMail');
    if (!defined $EMail || $EMail !~ /\S/) {
        $Form->addFormError({
            'Reason'=> 'NO_EMAIL',
            'Form'  => 'SaveRegistration',
            'Name'  => 'EMail',
        });
        $Form->addFormError({
            'Reason'=> 'NO_EMAIL',
            'Form'  => 'SaveRegistration',
            'Name'  => 'NewsletterCampaigns',
        });
        $Servlet->vars('NewsletterCampaigns', [map { {'Value' => $_} } @NewsletterCampaignIDs]);
        return;
    }

    my $FirstName = $Form->value('FirstName');
    my $LastName = $Form->value('LastName');
    if ((!defined $FirstName || $FirstName !~ /\S/) && (!defined $LastName || $LastName !~ /\S/)) {
        $Form->addFormError({
            'Reason'=> 'NO_NAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'FirstName',
        });
        $Form->addFormError({
            'Reason'=> 'NO_NAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'LastName',
        });
        $Form->addFormError({
            'Reason'=> 'NO_NAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'NewsletterCampaigns',
        });
        $Servlet->vars('NewsletterCampaigns', [map { {'Value' => $_} } @NewsletterCampaignIDs]);
        return;
    }

    my $Name = $FirstName;
    $Name = defined $Name ? $Name . ' ' . $LastName : $LastName;
    my $Shop = $Servlet->object->getSite;

    foreach my $NewsletterCampaignID (@NewsletterCampaignIDs) {
        my $NewsletterCampaign = $Shop->loadObject($NewsletterCampaignID, 'NewsletterCampaign');
        eval {
            if (!$NewsletterCampaign->existsAddressee($EMail)) {
                $NewsletterCampaign->addAddressee({
                    'EMail' => $EMail,
                    'Name' => $Name,
                    'Unsubscribed' => 2,
                    'SubscribedInCheckout' => 1
                });
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ( $Error->code eq 'FORMAT_NOT_EMAILADDRESS' ) {
                $Form->addFormError({
                    'Reason'=> 'FORMAT_NOT_EMAILADDRESS',
                    'Name'  => 'EMail',
                });
            }
            elsif ( $Error->code eq 'STRING_TOO_LONG' ) {
                $Form->addFormError({
                    'Reason'=> 'STRING_TOO_LONG',
                    'Name'  => 'FirstName',
                });
                $Form->addFormError({
                    'Reason'=> 'STRING_TOO_LONG',
                    'Name'  => 'LastName',
                });
            }
            else {
                Error($Error);
            }
        }
    }
}

#========================================================================================
# function     OnAfterSaveRegistration
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterSaveRegistration( $hParams );
#----------------------------------------------------------------------------------------
# description  Set user to new addressee. if no Double-Opt-In, sets Unsibscribed=0 and
#               remove SubscribedInCheckout.
#               (Hook UI_AfterSaveRegistration)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet</li>
#                   <li>User - user object</li>
#                   <li>Customer - customer object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterSaveRegistration {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $User = $hParams->{'User'};
    my $Shop = $Servlet->object->getSite();
    my $EMail = $Servlet->form->value('EMail');
    my @aSelectedCampaignIDs = $Servlet->form->value('Campaign');
    my @aNewsletterCampaignIDs = @{$Shop->get('NewsletterCampaigns')};

    @aNewsletterCampaignIDs = grep {$_->id ~~ @aSelectedCampaignIDs} @aNewsletterCampaignIDs;

    return unless defined $EMail;
    return if 0 == scalar @aNewsletterCampaignIDs;

    my $UnSubscribe = 0;
    my $NewsletterUIPackage  = DE_EPAGES::Newsletter::UI::Site->new({'Servlet'} => $Servlet);
    $NewsletterUIPackage ->SendConfirmationRequestMail($Servlet, $User->get('Name'), $User->get('EMail'),
        $UnSubscribe, \@aNewsletterCampaignIDs);

    return;
}

#========================================================================================
# function     OnConfirmUserAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnConfirmUserAccount( $hParams );
#----------------------------------------------------------------------------------------
# description  Set user to new addressee. (Hook UI_ConfirmUserAccount)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet</li>
#                   <li>User - user object</li>
#                   <li>EMail - email address</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnConfirmUserAccount {
    my ($hParams) = @_;
    my $User = $hParams->{'User'};
    my $Shop = $User->getSite();
    my $EMail = $hParams->{'EMail'};

    foreach my $NewsletterCampaign (@{ $Shop->get('NewsletterCampaigns') }){
        if ($NewsletterCampaign->existsAddressee($EMail)) {
            my $Addressee = $NewsletterCampaign->getAddressee($EMail);
            if ($Addressee->get('SubscribedInCheckout') && $Addressee->get('Unsubscribed') == 2) {
                $Addressee->set({'User' => $User}) unless defined $Addressee->get('User');
                $Addressee->set({
                    'SubscribedInCheckout' => 0
                });
            }
        }
    }
}


1;
