#========================================================================================
# package      DE_EPAGES::Tax::API::Cartridge
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Tax::API::Cartridge;
use base DE_EPAGES::Presentation::API::PresentationInstaller;

use strict;
use DE_EPAGES::Core::API::File qw( RunOnGlob );
use DE_EPAGES::Object::API::Factory qw( Transaction );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw (DeleteObjects);
use DE_EPAGES::XML::API::TestSchema qw ( TestSchema );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Tax::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Tax.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                   cartridges
#               </ul>
#               | hash
#========================================================================================
sub new {
    my ($class, %options) = @_;
    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory'=> 'DE_EPAGES/Tax',
        'Version'            => '7.26.0',
        'Patches'           => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;
    $self->startInstall;
    $self->SUPER::install;

    Transaction( sub {
        my $XMLDir = $self->xmlDirectory;
        RunOnGlob($XMLDir, 'Taxes*.xml', sub { ImportObjects( $_, '/' ) });
        ImportObjects( "$XMLDir/TaxMatrixDigital.xml" );
        ImportObjects( "$XMLDir/DefaultTaxMatrix.xml" );
        ImportObjects( "$XMLDir/DefaultTaxMatrixDigital.xml" );
        ImportObjects( "$XMLDir/TaxTypes.xml" );
        ImportTranslation( "$XMLDir/Translation.TaxAreasDigital" );
    });

    $self->finishInstall;
    return;
}

#========================================================================================
# function     test
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test;
#----------------------------------------------------------------------------------------
# description  runs xml schema validation for Taxes
#========================================================================================
sub test {
    my $self = shift;
    $self->SUPER::test();
    my $XMLDir = $self->xmlDirectory;
    my $XSDDir = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Tax/Data/XSD/';
    TestSchema($XMLDir, 'Taxes*.xml', [$XSDDir.'Taxes.xsd', $XSDDir.'v6/Taxes.xsd'] );
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;
    $self->startUninstall;

    my $XMLDir = $self->xmlDirectory;
    RunOnGlob($XMLDir, 'Taxes*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force});
    DeleteObjects("$XMLDir/TaxTypes.xml");

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_6_17_11
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_11;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.11' to version '6.17.12'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.12' | string
#========================================================================================
sub patchDBStructure_6_17_11 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

    }

    $self->importPatchDir('Patch_6_17_11', 'PatchDBStructure');

    return '6.17.12';
}

1;
